# automatically generated by the FlatBuffers compiler, do not modify

# namespace: MyGame

import flatbuffers
from flatbuffers.compat import import_numpy
from typing import Any
np = import_numpy()

class MonsterExtra(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset: int = 0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = MonsterExtra()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsMonsterExtra(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    @classmethod
    def MonsterExtraBufferHasIdentifier(cls, buf, offset, size_prefixed=False):
        return flatbuffers.util.BufferHasIdentifier(buf, offset, b"\x4D\x4F\x4E\x45", size_prefixed=size_prefixed)

    # MonsterExtra
    def Init(self, buf: bytes, pos: int):
        self._tab = flatbuffers.table.Table(buf, pos)

    # MonsterExtra
    def D0(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return float('nan')

    # MonsterExtra
    def D1(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return float('nan')

    # MonsterExtra
    def D2(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return float('inf')

    # MonsterExtra
    def D3(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return float('-inf')

    # MonsterExtra
    def F0(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float32Flags, o + self._tab.Pos)
        return float('nan')

    # MonsterExtra
    def F1(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float32Flags, o + self._tab.Pos)
        return float('nan')

    # MonsterExtra
    def F2(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float32Flags, o + self._tab.Pos)
        return float('inf')

    # MonsterExtra
    def F3(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float32Flags, o + self._tab.Pos)
        return float('-inf')

    # MonsterExtra
    def Dvec(self, j: int):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Float64Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 8))
        return 0

    # MonsterExtra
    def DvecAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Float64Flags, o)
        return 0

    # MonsterExtra
    def DvecLength(self) -> int:
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # MonsterExtra
    def DvecIsNone(self) -> bool:
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        return o == 0

    # MonsterExtra
    def Fvec(self, j: int):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(22))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.Get(flatbuffers.number_types.Float32Flags, a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 4))
        return 0

    # MonsterExtra
    def FvecAsNumpy(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(22))
        if o != 0:
            return self._tab.GetVectorAsNumpy(flatbuffers.number_types.Float32Flags, o)
        return 0

    # MonsterExtra
    def FvecLength(self) -> int:
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(22))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # MonsterExtra
    def FvecIsNone(self) -> bool:
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(22))
        return o == 0

def MonsterExtraStart(builder: flatbuffers.Builder):
    builder.StartObject(11)

def Start(builder: flatbuffers.Builder):
    MonsterExtraStart(builder)

def MonsterExtraAddD0(builder: flatbuffers.Builder, d0: float):
    builder.PrependFloat64Slot(0, d0, float('nan'))

def AddD0(builder: flatbuffers.Builder, d0: float):
    MonsterExtraAddD0(builder, d0)

def MonsterExtraAddD1(builder: flatbuffers.Builder, d1: float):
    builder.PrependFloat64Slot(1, d1, float('nan'))

def AddD1(builder: flatbuffers.Builder, d1: float):
    MonsterExtraAddD1(builder, d1)

def MonsterExtraAddD2(builder: flatbuffers.Builder, d2: float):
    builder.PrependFloat64Slot(2, d2, float('inf'))

def AddD2(builder: flatbuffers.Builder, d2: float):
    MonsterExtraAddD2(builder, d2)

def MonsterExtraAddD3(builder: flatbuffers.Builder, d3: float):
    builder.PrependFloat64Slot(3, d3, float('-inf'))

def AddD3(builder: flatbuffers.Builder, d3: float):
    MonsterExtraAddD3(builder, d3)

def MonsterExtraAddF0(builder: flatbuffers.Builder, f0: float):
    builder.PrependFloat32Slot(4, f0, float('nan'))

def AddF0(builder: flatbuffers.Builder, f0: float):
    MonsterExtraAddF0(builder, f0)

def MonsterExtraAddF1(builder: flatbuffers.Builder, f1: float):
    builder.PrependFloat32Slot(5, f1, float('nan'))

def AddF1(builder: flatbuffers.Builder, f1: float):
    MonsterExtraAddF1(builder, f1)

def MonsterExtraAddF2(builder: flatbuffers.Builder, f2: float):
    builder.PrependFloat32Slot(6, f2, float('inf'))

def AddF2(builder: flatbuffers.Builder, f2: float):
    MonsterExtraAddF2(builder, f2)

def MonsterExtraAddF3(builder: flatbuffers.Builder, f3: float):
    builder.PrependFloat32Slot(7, f3, float('-inf'))

def AddF3(builder: flatbuffers.Builder, f3: float):
    MonsterExtraAddF3(builder, f3)

def MonsterExtraAddDvec(builder: flatbuffers.Builder, dvec: int):
    builder.PrependUOffsetTRelativeSlot(8, flatbuffers.number_types.UOffsetTFlags.py_type(dvec), 0)

def AddDvec(builder: flatbuffers.Builder, dvec: int):
    MonsterExtraAddDvec(builder, dvec)

def MonsterExtraStartDvecVector(builder, numElems: int) -> int:
    return builder.StartVector(8, numElems, 8)

def StartDvecVector(builder, numElems: int) -> int:
    return MonsterExtraStartDvecVector(builder, numElems)

def MonsterExtraAddFvec(builder: flatbuffers.Builder, fvec: int):
    builder.PrependUOffsetTRelativeSlot(9, flatbuffers.number_types.UOffsetTFlags.py_type(fvec), 0)

def AddFvec(builder: flatbuffers.Builder, fvec: int):
    MonsterExtraAddFvec(builder, fvec)

def MonsterExtraStartFvecVector(builder, numElems: int) -> int:
    return builder.StartVector(4, numElems, 4)

def StartFvecVector(builder, numElems: int) -> int:
    return MonsterExtraStartFvecVector(builder, numElems)

def MonsterExtraEnd(builder: flatbuffers.Builder) -> int:
    return builder.EndObject()

def End(builder: flatbuffers.Builder) -> int:
    return MonsterExtraEnd(builder)

try:
    from typing import List
except:
    pass

class MonsterExtraT(object):

    # MonsterExtraT
    def __init__(
        self,
        d0 = float('nan'),
        d1 = float('nan'),
        d2 = float('inf'),
        d3 = float('-inf'),
        f0 = float('nan'),
        f1 = float('nan'),
        f2 = float('inf'),
        f3 = float('-inf'),
        dvec = None,
        fvec = None,
    ):
        self.d0 = d0  # type: float
        self.d1 = d1  # type: float
        self.d2 = d2  # type: float
        self.d3 = d3  # type: float
        self.f0 = f0  # type: float
        self.f1 = f1  # type: float
        self.f2 = f2  # type: float
        self.f3 = f3  # type: float
        self.dvec = dvec  # type: Optional[List[float]]
        self.fvec = fvec  # type: Optional[List[float]]

    @classmethod
    def InitFromBuf(cls, buf, pos):
        monsterExtra = MonsterExtra()
        monsterExtra.Init(buf, pos)
        return cls.InitFromObj(monsterExtra)

    @classmethod
    def InitFromPackedBuf(cls, buf, pos=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, pos)
        return cls.InitFromBuf(buf, pos+n)

    @classmethod
    def InitFromObj(cls, monsterExtra):
        x = MonsterExtraT()
        x._UnPack(monsterExtra)
        return x

    def __eq__(self, other):
        return type(self) == type(other) and \
            self.d0 == other.d0 and \
            self.d1 == other.d1 and \
            self.d2 == other.d2 and \
            self.d3 == other.d3 and \
            self.f0 == other.f0 and \
            self.f1 == other.f1 and \
            self.f2 == other.f2 and \
            self.f3 == other.f3 and \
            np.array_equal(self.dvec, other.dvec) and \
            np.array_equal(self.fvec, other.fvec)

    # MonsterExtraT
    def _UnPack(self, monsterExtra):
        if monsterExtra is None:
            return
        self.d0 = monsterExtra.D0()
        self.d1 = monsterExtra.D1()
        self.d2 = monsterExtra.D2()
        self.d3 = monsterExtra.D3()
        self.f0 = monsterExtra.F0()
        self.f1 = monsterExtra.F1()
        self.f2 = monsterExtra.F2()
        self.f3 = monsterExtra.F3()
        if not monsterExtra.DvecIsNone():
            if np is None:
                self.dvec = []
                for i in range(monsterExtra.DvecLength()):
                    self.dvec.append(monsterExtra.Dvec(i))
            else:
                self.dvec = monsterExtra.DvecAsNumpy()
        if not monsterExtra.FvecIsNone():
            if np is None:
                self.fvec = []
                for i in range(monsterExtra.FvecLength()):
                    self.fvec.append(monsterExtra.Fvec(i))
            else:
                self.fvec = monsterExtra.FvecAsNumpy()

    # MonsterExtraT
    def Pack(self, builder):
        if self.dvec is not None:
            if np is not None and type(self.dvec) is np.ndarray:
                dvec = builder.CreateNumpyVector(self.dvec)
            else:
                MonsterExtraStartDvecVector(builder, len(self.dvec))
                for i in reversed(range(len(self.dvec))):
                    builder.PrependFloat64(self.dvec[i])
                dvec = builder.EndVector()
        if self.fvec is not None:
            if np is not None and type(self.fvec) is np.ndarray:
                fvec = builder.CreateNumpyVector(self.fvec)
            else:
                MonsterExtraStartFvecVector(builder, len(self.fvec))
                for i in reversed(range(len(self.fvec))):
                    builder.PrependFloat32(self.fvec[i])
                fvec = builder.EndVector()
        MonsterExtraStart(builder)
        MonsterExtraAddD0(builder, self.d0)
        MonsterExtraAddD1(builder, self.d1)
        MonsterExtraAddD2(builder, self.d2)
        MonsterExtraAddD3(builder, self.d3)
        MonsterExtraAddF0(builder, self.f0)
        MonsterExtraAddF1(builder, self.f1)
        MonsterExtraAddF2(builder, self.f2)
        MonsterExtraAddF3(builder, self.f3)
        if self.dvec is not None:
            MonsterExtraAddDvec(builder, dvec)
        if self.fvec is not None:
            MonsterExtraAddFvec(builder, fvec)
        monsterExtra = MonsterExtraEnd(builder)
        return monsterExtra



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: NestedUnion

class Any(object):
    NONE = 0
    Vec3 = 1
    TestSimpleTableWithEnum = 2

def AnyCreator(unionType, table):
    from flatbuffers.table import Table
    if not isinstance(table, Table):
        return None
    if unionType == Any.Vec3:
        import MyGame.Example.NestedUnion.Vec3
        return MyGame.Example.NestedUnion.Vec3.Vec3T.InitFromBuf(table.Bytes, table.Pos)
    if unionType == Any.TestSimpleTableWithEnum:
        import MyGame.Example.NestedUnion.TestSimpleTableWithEnum
        return MyGame.Example.NestedUnion.TestSimpleTableWithEnum.TestSimpleTableWithEnumT.InitFromBuf(table.Bytes, table.Pos)
    return None



#
# Copyright 2013 The py-lmdb authors, all rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted only as authorized by the OpenLDAP
# Public License.
#
# A copy of this license is available in the file LICENSE in the
# top-level directory of the distribution or, alternatively, at
# <http://www.OpenLDAP.org/license.html>.
#
# OpenLDAP is a registered trademark of the OpenLDAP Foundation.
#
# Individual files and/or contributed packages may be copyright by
# other parties and/or subject to additional restrictions.
#
# This work also contains materials derived from public sources.
#
# Additional information about OpenLDAP can be obtained at
# <http://www.openldap.org/>.
#

from __future__ import absolute_import
import unittest

import zlmdb.lmdb as lmdb


class PackageExportsTest(unittest.TestCase):
    """
    Ensure the list of exported names matches a predefined list. Designed to
    ensure future interface changes to cffi.py and cpython.c don't break
    consistency of "from lmdb import *".
    """

    def test_exports(self):
        assert sorted(lmdb.__all__) == [
            "BadDbiError",
            "BadRslotError",
            "BadTxnError",
            "BadValsizeError",
            "CorruptedError",
            "Cursor",
            "CursorFullError",
            "DbsFullError",
            "DiskError",
            "Environment",
            "Error",
            "IncompatibleError",
            "InvalidError",
            "InvalidParameterError",
            "KeyExistsError",
            "LockError",
            "MapFullError",
            "MapResizedError",
            "MemoryError",
            "NotFoundError",
            "PageFullError",
            "PageNotFoundError",
            "PanicError",
            "ReadersFullError",
            "ReadonlyError",
            "TlsFullError",
            "Transaction",
            "TxnFullError",
            "VersionMismatchError",
            "_Database",
            "enable_drop_gil",
            "version",
        ]



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: NamespaceA

import flatbuffers
from flatbuffers.compat import import_numpy

np = import_numpy()


class SecondTableInA(object):
  __slots__ = ['_tab']

  @classmethod
  def GetRootAs(cls, buf, offset=0):
    n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
    x = SecondTableInA()
    x.Init(buf, n + offset)
    return x

  @classmethod
  def GetRootAsSecondTableInA(cls, buf, offset=0):
    """This method is deprecated. Please switch to GetRootAs."""
    return cls.GetRootAs(buf, offset)

  # SecondTableInA
  def Init(self, buf, pos):
    self._tab = flatbuffers.table.Table(buf, pos)

  # SecondTableInA
  def ReferToC(self):
    o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
    if o != 0:
      x = self._tab.Indirect(o + self._tab.Pos)
      obj = TableInC()
      obj.Init(self._tab.Bytes, x)
      return obj
    return None


def SecondTableInAStart(builder):
  builder.StartObject(1)


def Start(builder):
  return SecondTableInAStart(builder)


def SecondTableInAAddReferToC(builder, referToC):
  builder.PrependUOffsetTRelativeSlot(
      0, flatbuffers.number_types.UOffsetTFlags.py_type(referToC), 0
  )


def AddReferToC(builder, referToC):
  return SecondTableInAAddReferToC(builder, referToC)


def SecondTableInAEnd(builder):
  return builder.EndObject()


def End(builder):
  return SecondTableInAEnd(builder)


try:
  from typing import Optional
except:
  pass


class SecondTableInAT(object):

  # SecondTableInAT
  def __init__(self):
    self.referToC = None  # type: Optional[TableInCT]

  @classmethod
  def InitFromBuf(cls, buf, pos):
    secondTableInA = SecondTableInA()
    secondTableInA.Init(buf, pos)
    return cls.InitFromObj(secondTableInA)

  @classmethod
  def InitFromObj(cls, secondTableInA):
    x = SecondTableInAT()
    x._UnPack(secondTableInA)
    return x

  # SecondTableInAT
  def _UnPack(self, secondTableInA):
    if secondTableInA is None:
      return
    if secondTableInA.ReferToC() is not None:
      self.referToC = TableInCT.InitFromObj(secondTableInA.ReferToC())

  # SecondTableInAT
  def Pack(self, builder):
    if self.referToC is not None:
      referToC = self.referToC.Pack(builder)
    SecondTableInAStart(builder)
    if self.referToC is not None:
      SecondTableInAAddReferToC(builder, referToC)
    secondTableInA = SecondTableInAEnd(builder)
    return secondTableInA



"""
Hatchling custom build hook for CFFI extension modules.

This builds the vendored LMDB CFFI extension by:
1. Running build_lmdb.py to prepare LMDB sources (patches from git submodule)
2. Running build_cffi_lmdb.py to compile the CFFI extension

See: https://hatch.pypa.io/latest/plugins/build-hook/custom/
"""

import os
import sys
import subprocess
import sysconfig
from pathlib import Path

from hatchling.builders.hooks.plugin.interface import BuildHookInterface


class LmdbCffiBuildHook(BuildHookInterface):
    """Build hook for compiling LMDB CFFI extension module."""

    PLUGIN_NAME = "lmdb-cffi"

    def initialize(self, version, build_data):
        """
        Called before each build.

        For wheel builds, compile the CFFI modules.
        For sdist builds, just ensure source files are included.
        """
        if self.target_name != "wheel":
            # Only compile for wheel builds, sdist just includes source
            return

        # Build CFFI module
        built_extension = self._build_lmdb_cffi(build_data)

        # If we built extension, mark this as a platform-specific wheel
        if built_extension:
            build_data["infer_tag"] = True
            build_data["pure_python"] = False

    def _build_lmdb_cffi(self, build_data):
        """Build the LMDB CFFI extension module.

        Returns True if extension was successfully built.
        """
        # Get the extension suffix for current Python to filter artifacts
        ext_suffix = sysconfig.get_config_var("EXT_SUFFIX") or ".so"
        print(f"Building LMDB CFFI for Python with extension suffix: {ext_suffix}")

        # Step 1: Run build_lmdb.py to prepare LMDB sources
        print("=" * 70)
        print("Step 1: Preparing LMDB sources (build_lmdb.py)")
        print("=" * 70)

        build_lmdb_script = Path(self.root) / "build_lmdb.py"
        if not build_lmdb_script.exists():
            print(f"ERROR: {build_lmdb_script} not found")
            return False

        result = subprocess.run(
            [sys.executable, str(build_lmdb_script)],
            cwd=self.root,
            capture_output=False,
        )
        if result.returncode != 0:
            print("ERROR: build_lmdb.py failed!")
            return False

        # Step 2: Run build_cffi_lmdb.py to compile CFFI extension
        print("\n" + "=" * 70)
        print("Step 2: Compiling CFFI extension (build_cffi_lmdb.py)")
        print("=" * 70)

        build_cffi_script = Path(self.root) / "build_cffi_lmdb.py"
        if not build_cffi_script.exists():
            print(f"ERROR: {build_cffi_script} not found")
            return False

        result = subprocess.run(
            [sys.executable, str(build_cffi_script)],
            cwd=self.root,
            capture_output=False,
        )
        if result.returncode != 0:
            print("ERROR: build_cffi_lmdb.py failed!")
            return False

        # Step 3: Find the compiled artifact and add to wheel
        print("\n" + "=" * 70)
        print("Step 3: Adding compiled extension to wheel")
        print("=" * 70)

        vendor_dir = Path(self.root) / "src" / "zlmdb" / "_lmdb_vendor"
        expected_so = vendor_dir / f"_lmdb_cffi{ext_suffix}"

        if expected_so.exists():
            src_file = str(expected_so)
            dest_path = f"zlmdb/_lmdb_vendor/{expected_so.name}"
            build_data["force_include"][src_file] = dest_path
            print(f"  -> Added artifact: {expected_so.name} -> {dest_path}")
            return True
        else:
            print(f"WARNING: Expected CFFI extension not found: {expected_so}")
            # Check what .so files exist
            for so_file in vendor_dir.glob("_lmdb_cffi*.so"):
                print(f"  Found: {so_file.name}")
            return False



from golden_utils import flatc_golden


def flatc(options, schema):
  # Wrap the golden flatc generator with Python specifics
  flatc_golden(options=["--python"] + options, schema=schema, prefix="py")


def GeneratePython():
  flatc([], "basic.fbs")



import random
import zlmdb


class Foo(object):

    oid: int
    value: float
    msg: str

    def __init__(self, oid=None, value=None, msg=None):
        self.oid = oid
        self.value = value
        self.msg = msg

    @staticmethod
    def unmarshal(obj):
        return Foo(obj['oid'], obj['value'], obj['msg'])

    def marshal(self):
        return {'oid': self.oid, 'value': self.value, 'msg': self.msg}


class MySchema(zlmdb.Schema):

    tab1: zlmdb.MapOidPickle = zlmdb.MapOidPickle(1)
    # tab1: zlmdb.MapOidJson = zlmdb.MapOidJson(1, marshal=Foo.marshal, unmarshal=Foo.unmarshal)
    # tab1: zlmdb.MapOidCbor = zlmdb.MapOidCbor(1, marshal=Foo.marshal, unmarshal=Foo.unmarshal)


schema = MySchema()

with schema.open('.testdb') as db:

    with db.begin(write=True) as txn:

        o1 = Foo(23, random.random(), 'Hello, world!')

        schema.tab1[txn, o1.oid] = o1
        print('object saved:', o1.oid, o1.value, o1.msg)

        o2 = schema.tab1[txn, o1.oid]
        assert o2
        print('object loaded:', o2.oid, o2.value, o2.msg)

    print('transaction committed')

print('database closed')



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: Example2

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class Monster(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Monster()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsMonster(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    @classmethod
    def MonsterBufferHasIdentifier(cls, buf, offset, size_prefixed=False):
        return flatbuffers.util.BufferHasIdentifier(buf, offset, b"\x4D\x4F\x4E\x53", size_prefixed=size_prefixed)

    # Monster
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

def MonsterStart(builder):
    builder.StartObject(0)

def Start(builder):
    MonsterStart(builder)

def MonsterEnd(builder):
    return builder.EndObject()

def End(builder):
    return MonsterEnd(builder)


class MonsterT(object):

    # MonsterT
    def __init__(
        self,
    ):
        pass

    @classmethod
    def InitFromBuf(cls, buf, pos):
        monster = Monster()
        monster.Init(buf, pos)
        return cls.InitFromObj(monster)

    @classmethod
    def InitFromPackedBuf(cls, buf, pos=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, pos)
        return cls.InitFromBuf(buf, pos+n)

    @classmethod
    def InitFromObj(cls, monster):
        x = MonsterT()
        x._UnPack(monster)
        return x

    # MonsterT
    def _UnPack(self, monster):
        if monster is None:
            return

    # MonsterT
    def Pack(self, builder):
        MonsterStart(builder)
        monster = MonsterEnd(builder)
        return monster



from __future__ import annotations

import flatbuffers
import numpy as np

import typing
from MyGame.Example.NestedUnion.Color import Color

uoffset: typing.TypeAlias = flatbuffers.number_types.UOffsetTFlags.py_type

class TestSimpleTableWithEnum(object):
  @classmethod
  def GetRootAs(cls, buf: bytes, offset: int) -> TestSimpleTableWithEnum: ...
  @classmethod
  def GetRootAsTestSimpleTableWithEnum(cls, buf: bytes, offset: int) -> TestSimpleTableWithEnum: ...
  def Init(self, buf: bytes, pos: int) -> None: ...
  def Color(self) -> typing.Literal[Color.Red, Color.Green, Color.Blue]: ...
class TestSimpleTableWithEnumT(object):
  color: typing.Literal[Color.Red, Color.Green, Color.Blue]
  def __init__(
    self,
    color: typing.Literal[Color.Red, Color.Green, Color.Blue] = ...,
  ) -> None: ...
  @classmethod
  def InitFromBuf(cls, buf: bytes, pos: int) -> TestSimpleTableWithEnumT: ...
  @classmethod
  def InitFromPackedBuf(cls, buf: bytes, pos: int = 0) -> TestSimpleTableWithEnumT: ...
  @classmethod
  def InitFromObj(cls, testSimpleTableWithEnum: TestSimpleTableWithEnum) -> TestSimpleTableWithEnumT: ...
  def _UnPack(self, testSimpleTableWithEnum: TestSimpleTableWithEnum) -> None: ...
  def Pack(self, builder: flatbuffers.Builder) -> None: ...
def TestSimpleTableWithEnumStart(builder: flatbuffers.Builder) -> None: ...
def Start(builder: flatbuffers.Builder) -> None: ...
def TestSimpleTableWithEnumAddColor(builder: flatbuffers.Builder, color: typing.Literal[Color.Red, Color.Green, Color.Blue]) -> None: ...
def TestSimpleTableWithEnumEnd(builder: flatbuffers.Builder) -> uoffset: ...
def End(builder: flatbuffers.Builder) -> uoffset: ...




# docs/conf.py
# zlmdb documentation configuration - modernized for 2025
import os
import sys
from datetime import datetime

# -- Path setup --------------------------------------------------------------
sys.path.insert(0, os.path.abspath("../src"))

# Add .cicd/scripts to path for shared Sphinx extensions
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '.cicd', 'scripts')))

# monkey-patch txaio so that we can "use" both twisted *and* asyncio,
# at least at import time
import txaio


def use_tx():
    "monkey-patched for doc-building"
    from txaio import tx

    txaio._use_framework(tx)


def use_aio():
    "monkey-patched for doc-building"
    from txaio import aio

    txaio._use_framework(aio)


txaio.use_twisted = use_tx
txaio.use_asyncio = use_aio

# -- Project information -----------------------------------------------------
project = "zlmdb"
author = "The WAMP/Autobahn/Crossbar.io OSS Project"
copyright = f"2018-{datetime.now():%Y}, typedef int GmbH (Germany)"
language = "en"

# Get version from the package
base_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir))
with open(os.path.join(base_dir, "src", "zlmdb", "_version.py")) as f:
    exec(f.read())  # defines __version__

version = release = __version__  # noqa

# -- General configuration ---------------------------------------------------
extensions = [
    # MyST Markdown support
    "myst_parser",

    # Core Sphinx extensions
    "sphinx.ext.autodoc",
    "sphinx.ext.autodoc.typehints",
    "sphinx.ext.napoleon",
    "sphinx.ext.intersphinx",
    "sphinx.ext.autosectionlabel",
    "sphinx.ext.todo",
    "sphinx.ext.viewcode",
    "sphinx.ext.ifconfig",
    "sphinx.ext.doctest",
    "sphinx.ext.inheritance_diagram",

    # Modern UX extensions
    "sphinx_design",
    "sphinx_copybutton",
    "sphinxext.opengraph",
    "sphinxcontrib.images",
    "sphinxcontrib.spelling",

    # API documentation
    "autoapi.extension",

    # Shared WAMP ecosystem extensions (from .cicd submodule)
    "sphinx_auto_section_anchors",   # Stable slug-based HTML anchors
]

# Source file suffixes (both RST and MyST Markdown)
source_suffix = {
    ".rst": "restructuredtext",
    ".md": "markdown",
}

# The master toctree document
master_doc = "index"

# Exclude patterns
exclude_patterns = ["_build", "README.md"]

# -- MyST Configuration ------------------------------------------------------
myst_enable_extensions = [
    "colon_fence",
    "deflist",
    "tasklist",
    "attrs_block",
    "attrs_inline",
    "smartquotes",
    "linkify",
]
myst_heading_anchors = 3

# -- AutoAPI Configuration ---------------------------------------------------
autoapi_type = "python"
autoapi_dirs = ["../src/zlmdb"]
autoapi_add_toctree_entry = True
autoapi_keep_files = False              # Cleaner RTD builds
autoapi_generate_api_docs = True
autoapi_options = [
    "members",
    "undoc-members",
    "private-members",
    "special-members",
    "show-inheritance",
    "show-module-summary",
    "imported-members",
]
autoapi_ignore = [
    "*/_version.py",
    "*/test_*.py",
    "*/*_test.py",
    "*/conftest.py",
]
autoapi_python_use_implicit_namespaces = True
autoapi_member_order = "alphabetical"   # Predictable ordering

# -- Intersphinx Configuration -----------------------------------------------
intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
    "twisted": ("https://docs.twisted.org/en/stable/", None),
    "txaio": ("https://txaio.readthedocs.io/en/latest/", None),
    "autobahn": ("https://autobahn.readthedocs.io/en/latest/", None),
    "numpy": ("https://numpy.org/doc/stable/", None),
}
intersphinx_cache_limit = 5

# -- HTML Output (Furo Theme) ------------------------------------------------
html_theme = "furo"
html_title = f"{project} {release}"

# Furo theme options with Noto fonts and Autobahn subarea colors
html_theme_options = {
    # Source repository links
    "source_repository": "https://github.com/crossbario/zlmdb/",
    "source_branch": "master",
    "source_directory": "docs/",

    # Noto fonts and Autobahn Medium Blue (#027eae) accent color
    "light_css_variables": {
        "font-stack": "'Noto Sans', -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, sans-serif",
        "font-stack--monospace": "'Noto Sans Mono', SFMono-Regular, Menlo, Consolas, monospace",
        "color-brand-primary": "#027eae",
        "color-brand-content": "#027eae",
    },
    "dark_css_variables": {
        "font-stack": "'Noto Sans', -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, sans-serif",
        "font-stack--monospace": "'Noto Sans Mono', SFMono-Regular, Menlo, Consolas, monospace",
        "color-brand-primary": "#027eae",
        "color-brand-content": "#027eae",
    },
}

# Logo and favicon (synced from autobahn-python by `just sync-images`)
html_logo = "_static/img/autobahn_logo_blue.svg"
html_favicon = "_static/favicon.ico"

# Static files
html_static_path = ["_static"]
html_css_files = [
    # Load Noto fonts from Google Fonts
    "https://fonts.googleapis.com/css2?family=Noto+Sans:wght@400;500;600;700&family=Noto+Sans+Mono:wght@400;500&display=swap",
]

# -- sphinxcontrib-images Configuration --------------------------------------
# NOTE: override_image_directive must be False to preserve standard RST image
# directive :target: option support, which is required for clickable badge
# substitutions in docs/index.rst (e.g., |PyPI| |Python| |CI| etc.)
images_config = {
    "override_image_directive": False,
    "default_image_width": "80%",
}

# -- Spelling Configuration --------------------------------------------------
spelling_lang = "en_US"
spelling_word_list_filename = "spelling_wordlist.txt"
spelling_show_suggestions = True

# -- OpenGraph (Social Media Meta Tags) -------------------------------------
ogp_site_url = "https://zlmdb.readthedocs.io/en/latest/"

# -- Auto Section Anchors Configuration --------------------------------------
# Force overwrite of auto-generated ids (id1, id2, etc.) with slug-based anchors
auto_section_anchor_force = True

# -- Miscellaneous -----------------------------------------------------------
todo_include_todos = True
add_module_names = False
autosectionlabel_prefix_document = True
pygments_style = "sphinx"
pygments_dark_style = "monokai"



from __future__ import annotations

import flatbuffers

import typing

uoffset: typing.TypeAlias = flatbuffers.number_types.UOffsetTFlags.py_type

class HelloRequest(object):
  @classmethod
  def GetRootAs(cls, buf: bytes, offset: int) -> HelloRequest: ...
  @classmethod
  def GetRootAsHelloRequest(cls, buf: bytes, offset: int) -> HelloRequest: ...
  def Init(self, buf: bytes, pos: int) -> None: ...
def HelloRequestStart(builder: flatbuffers.Builder) -> None: ...
def HelloRequestEnd(builder: flatbuffers.Builder) -> uoffset: ...
class HelloResponse(object):
  @classmethod
  def GetRootAs(cls, buf: bytes, offset: int) -> HelloResponse: ...
  @classmethod
  def GetRootAsHelloResponse(cls, buf: bytes, offset: int) -> HelloResponse: ...
  def Init(self, buf: bytes, pos: int) -> None: ...
def HelloResponseStart(builder: flatbuffers.Builder) -> None: ...
def HelloResponseEnd(builder: flatbuffers.Builder) -> uoffset: ...




# automatically generated by the FlatBuffers compiler, do not modify

# namespace: optional_scalars

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class ScalarStuff(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = ScalarStuff()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsScalarStuff(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    @classmethod
    def ScalarStuffBufferHasIdentifier(cls, buf, offset, size_prefixed=False):
        return flatbuffers.util.BufferHasIdentifier(buf, offset, b"\x4E\x55\x4C\x4C", size_prefixed=size_prefixed)

    # ScalarStuff
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # ScalarStuff
    def JustI8(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int8Flags, o + self._tab.Pos)
        return 0

    # ScalarStuff
    def MaybeI8(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int8Flags, o + self._tab.Pos)
        return None

    # ScalarStuff
    def DefaultI8(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int8Flags, o + self._tab.Pos)
        return 42

    # ScalarStuff
    def JustU8(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, o + self._tab.Pos)
        return 0

    # ScalarStuff
    def MaybeU8(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, o + self._tab.Pos)
        return None

    # ScalarStuff
    def DefaultU8(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint8Flags, o + self._tab.Pos)
        return 42

    # ScalarStuff
    def JustI16(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int16Flags, o + self._tab.Pos)
        return 0

    # ScalarStuff
    def MaybeI16(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int16Flags, o + self._tab.Pos)
        return None

    # ScalarStuff
    def DefaultI16(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int16Flags, o + self._tab.Pos)
        return 42

    # ScalarStuff
    def JustU16(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(22))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint16Flags, o + self._tab.Pos)
        return 0

    # ScalarStuff
    def MaybeU16(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(24))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint16Flags, o + self._tab.Pos)
        return None

    # ScalarStuff
    def DefaultU16(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(26))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint16Flags, o + self._tab.Pos)
        return 42

    # ScalarStuff
    def JustI32(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(28))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int32Flags, o + self._tab.Pos)
        return 0

    # ScalarStuff
    def MaybeI32(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(30))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int32Flags, o + self._tab.Pos)
        return None

    # ScalarStuff
    def DefaultI32(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(32))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int32Flags, o + self._tab.Pos)
        return 42

    # ScalarStuff
    def JustU32(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(34))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return 0

    # ScalarStuff
    def MaybeU32(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(36))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return None

    # ScalarStuff
    def DefaultU32(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(38))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint32Flags, o + self._tab.Pos)
        return 42

    # ScalarStuff
    def JustI64(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(40))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int64Flags, o + self._tab.Pos)
        return 0

    # ScalarStuff
    def MaybeI64(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(42))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int64Flags, o + self._tab.Pos)
        return None

    # ScalarStuff
    def DefaultI64(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(44))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int64Flags, o + self._tab.Pos)
        return 42

    # ScalarStuff
    def JustU64(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(46))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint64Flags, o + self._tab.Pos)
        return 0

    # ScalarStuff
    def MaybeU64(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(48))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint64Flags, o + self._tab.Pos)
        return None

    # ScalarStuff
    def DefaultU64(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(50))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint64Flags, o + self._tab.Pos)
        return 42

    # ScalarStuff
    def JustF32(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(52))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float32Flags, o + self._tab.Pos)
        return 0.0

    # ScalarStuff
    def MaybeF32(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(54))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float32Flags, o + self._tab.Pos)
        return None

    # ScalarStuff
    def DefaultF32(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(56))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float32Flags, o + self._tab.Pos)
        return 42.0

    # ScalarStuff
    def JustF64(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(58))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # ScalarStuff
    def MaybeF64(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(60))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return None

    # ScalarStuff
    def DefaultF64(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(62))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 42.0

    # ScalarStuff
    def JustBool(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(64))
        if o != 0:
            return bool(self._tab.Get(flatbuffers.number_types.BoolFlags, o + self._tab.Pos))
        return False

    # ScalarStuff
    def MaybeBool(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(66))
        if o != 0:
            return bool(self._tab.Get(flatbuffers.number_types.BoolFlags, o + self._tab.Pos))
        return None

    # ScalarStuff
    def DefaultBool(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(68))
        if o != 0:
            return bool(self._tab.Get(flatbuffers.number_types.BoolFlags, o + self._tab.Pos))
        return True

    # ScalarStuff
    def JustEnum(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(70))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int8Flags, o + self._tab.Pos)
        return 0

    # ScalarStuff
    def MaybeEnum(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(72))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int8Flags, o + self._tab.Pos)
        return None

    # ScalarStuff
    def DefaultEnum(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(74))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int8Flags, o + self._tab.Pos)
        return 1

def ScalarStuffStart(builder):
    builder.StartObject(36)

def Start(builder):
    ScalarStuffStart(builder)

def ScalarStuffAddJustI8(builder, justI8):
    builder.PrependInt8Slot(0, justI8, 0)

def AddJustI8(builder, justI8):
    ScalarStuffAddJustI8(builder, justI8)

def ScalarStuffAddMaybeI8(builder, maybeI8):
    builder.PrependInt8Slot(1, maybeI8, None)

def AddMaybeI8(builder, maybeI8):
    ScalarStuffAddMaybeI8(builder, maybeI8)

def ScalarStuffAddDefaultI8(builder, defaultI8):
    builder.PrependInt8Slot(2, defaultI8, 42)

def AddDefaultI8(builder, defaultI8):
    ScalarStuffAddDefaultI8(builder, defaultI8)

def ScalarStuffAddJustU8(builder, justU8):
    builder.PrependUint8Slot(3, justU8, 0)

def AddJustU8(builder, justU8):
    ScalarStuffAddJustU8(builder, justU8)

def ScalarStuffAddMaybeU8(builder, maybeU8):
    builder.PrependUint8Slot(4, maybeU8, None)

def AddMaybeU8(builder, maybeU8):
    ScalarStuffAddMaybeU8(builder, maybeU8)

def ScalarStuffAddDefaultU8(builder, defaultU8):
    builder.PrependUint8Slot(5, defaultU8, 42)

def AddDefaultU8(builder, defaultU8):
    ScalarStuffAddDefaultU8(builder, defaultU8)

def ScalarStuffAddJustI16(builder, justI16):
    builder.PrependInt16Slot(6, justI16, 0)

def AddJustI16(builder, justI16):
    ScalarStuffAddJustI16(builder, justI16)

def ScalarStuffAddMaybeI16(builder, maybeI16):
    builder.PrependInt16Slot(7, maybeI16, None)

def AddMaybeI16(builder, maybeI16):
    ScalarStuffAddMaybeI16(builder, maybeI16)

def ScalarStuffAddDefaultI16(builder, defaultI16):
    builder.PrependInt16Slot(8, defaultI16, 42)

def AddDefaultI16(builder, defaultI16):
    ScalarStuffAddDefaultI16(builder, defaultI16)

def ScalarStuffAddJustU16(builder, justU16):
    builder.PrependUint16Slot(9, justU16, 0)

def AddJustU16(builder, justU16):
    ScalarStuffAddJustU16(builder, justU16)

def ScalarStuffAddMaybeU16(builder, maybeU16):
    builder.PrependUint16Slot(10, maybeU16, None)

def AddMaybeU16(builder, maybeU16):
    ScalarStuffAddMaybeU16(builder, maybeU16)

def ScalarStuffAddDefaultU16(builder, defaultU16):
    builder.PrependUint16Slot(11, defaultU16, 42)

def AddDefaultU16(builder, defaultU16):
    ScalarStuffAddDefaultU16(builder, defaultU16)

def ScalarStuffAddJustI32(builder, justI32):
    builder.PrependInt32Slot(12, justI32, 0)

def AddJustI32(builder, justI32):
    ScalarStuffAddJustI32(builder, justI32)

def ScalarStuffAddMaybeI32(builder, maybeI32):
    builder.PrependInt32Slot(13, maybeI32, None)

def AddMaybeI32(builder, maybeI32):
    ScalarStuffAddMaybeI32(builder, maybeI32)

def ScalarStuffAddDefaultI32(builder, defaultI32):
    builder.PrependInt32Slot(14, defaultI32, 42)

def AddDefaultI32(builder, defaultI32):
    ScalarStuffAddDefaultI32(builder, defaultI32)

def ScalarStuffAddJustU32(builder, justU32):
    builder.PrependUint32Slot(15, justU32, 0)

def AddJustU32(builder, justU32):
    ScalarStuffAddJustU32(builder, justU32)

def ScalarStuffAddMaybeU32(builder, maybeU32):
    builder.PrependUint32Slot(16, maybeU32, None)

def AddMaybeU32(builder, maybeU32):
    ScalarStuffAddMaybeU32(builder, maybeU32)

def ScalarStuffAddDefaultU32(builder, defaultU32):
    builder.PrependUint32Slot(17, defaultU32, 42)

def AddDefaultU32(builder, defaultU32):
    ScalarStuffAddDefaultU32(builder, defaultU32)

def ScalarStuffAddJustI64(builder, justI64):
    builder.PrependInt64Slot(18, justI64, 0)

def AddJustI64(builder, justI64):
    ScalarStuffAddJustI64(builder, justI64)

def ScalarStuffAddMaybeI64(builder, maybeI64):
    builder.PrependInt64Slot(19, maybeI64, None)

def AddMaybeI64(builder, maybeI64):
    ScalarStuffAddMaybeI64(builder, maybeI64)

def ScalarStuffAddDefaultI64(builder, defaultI64):
    builder.PrependInt64Slot(20, defaultI64, 42)

def AddDefaultI64(builder, defaultI64):
    ScalarStuffAddDefaultI64(builder, defaultI64)

def ScalarStuffAddJustU64(builder, justU64):
    builder.PrependUint64Slot(21, justU64, 0)

def AddJustU64(builder, justU64):
    ScalarStuffAddJustU64(builder, justU64)

def ScalarStuffAddMaybeU64(builder, maybeU64):
    builder.PrependUint64Slot(22, maybeU64, None)

def AddMaybeU64(builder, maybeU64):
    ScalarStuffAddMaybeU64(builder, maybeU64)

def ScalarStuffAddDefaultU64(builder, defaultU64):
    builder.PrependUint64Slot(23, defaultU64, 42)

def AddDefaultU64(builder, defaultU64):
    ScalarStuffAddDefaultU64(builder, defaultU64)

def ScalarStuffAddJustF32(builder, justF32):
    builder.PrependFloat32Slot(24, justF32, 0.0)

def AddJustF32(builder, justF32):
    ScalarStuffAddJustF32(builder, justF32)

def ScalarStuffAddMaybeF32(builder, maybeF32):
    builder.PrependFloat32Slot(25, maybeF32, None)

def AddMaybeF32(builder, maybeF32):
    ScalarStuffAddMaybeF32(builder, maybeF32)

def ScalarStuffAddDefaultF32(builder, defaultF32):
    builder.PrependFloat32Slot(26, defaultF32, 42.0)

def AddDefaultF32(builder, defaultF32):
    ScalarStuffAddDefaultF32(builder, defaultF32)

def ScalarStuffAddJustF64(builder, justF64):
    builder.PrependFloat64Slot(27, justF64, 0.0)

def AddJustF64(builder, justF64):
    ScalarStuffAddJustF64(builder, justF64)

def ScalarStuffAddMaybeF64(builder, maybeF64):
    builder.PrependFloat64Slot(28, maybeF64, None)

def AddMaybeF64(builder, maybeF64):
    ScalarStuffAddMaybeF64(builder, maybeF64)

def ScalarStuffAddDefaultF64(builder, defaultF64):
    builder.PrependFloat64Slot(29, defaultF64, 42.0)

def AddDefaultF64(builder, defaultF64):
    ScalarStuffAddDefaultF64(builder, defaultF64)

def ScalarStuffAddJustBool(builder, justBool):
    builder.PrependBoolSlot(30, justBool, 0)

def AddJustBool(builder, justBool):
    ScalarStuffAddJustBool(builder, justBool)

def ScalarStuffAddMaybeBool(builder, maybeBool):
    builder.PrependBoolSlot(31, maybeBool, None)

def AddMaybeBool(builder, maybeBool):
    ScalarStuffAddMaybeBool(builder, maybeBool)

def ScalarStuffAddDefaultBool(builder, defaultBool):
    builder.PrependBoolSlot(32, defaultBool, 1)

def AddDefaultBool(builder, defaultBool):
    ScalarStuffAddDefaultBool(builder, defaultBool)

def ScalarStuffAddJustEnum(builder, justEnum):
    builder.PrependInt8Slot(33, justEnum, 0)

def AddJustEnum(builder, justEnum):
    ScalarStuffAddJustEnum(builder, justEnum)

def ScalarStuffAddMaybeEnum(builder, maybeEnum):
    builder.PrependInt8Slot(34, maybeEnum, None)

def AddMaybeEnum(builder, maybeEnum):
    ScalarStuffAddMaybeEnum(builder, maybeEnum)

def ScalarStuffAddDefaultEnum(builder, defaultEnum):
    builder.PrependInt8Slot(35, defaultEnum, 1)

def AddDefaultEnum(builder, defaultEnum):
    ScalarStuffAddDefaultEnum(builder, defaultEnum)

def ScalarStuffEnd(builder):
    return builder.EndObject()

def End(builder):
    return ScalarStuffEnd(builder)

try:
    from typing import Optional
except:
    pass

class ScalarStuffT(object):

    # ScalarStuffT
    def __init__(
        self,
        justI8 = 0,
        maybeI8 = None,
        defaultI8 = 42,
        justU8 = 0,
        maybeU8 = None,
        defaultU8 = 42,
        justI16 = 0,
        maybeI16 = None,
        defaultI16 = 42,
        justU16 = 0,
        maybeU16 = None,
        defaultU16 = 42,
        justI32 = 0,
        maybeI32 = None,
        defaultI32 = 42,
        justU32 = 0,
        maybeU32 = None,
        defaultU32 = 42,
        justI64 = 0,
        maybeI64 = None,
        defaultI64 = 42,
        justU64 = 0,
        maybeU64 = None,
        defaultU64 = 42,
        justF32 = 0.0,
        maybeF32 = None,
        defaultF32 = 42.0,
        justF64 = 0.0,
        maybeF64 = None,
        defaultF64 = 42.0,
        justBool = False,
        maybeBool = None,
        defaultBool = True,
        justEnum = 0,
        maybeEnum = None,
        defaultEnum = 1,
    ):
        self.justI8 = justI8  # type: int
        self.maybeI8 = maybeI8  # type: Optional[int]
        self.defaultI8 = defaultI8  # type: int
        self.justU8 = justU8  # type: int
        self.maybeU8 = maybeU8  # type: Optional[int]
        self.defaultU8 = defaultU8  # type: int
        self.justI16 = justI16  # type: int
        self.maybeI16 = maybeI16  # type: Optional[int]
        self.defaultI16 = defaultI16  # type: int
        self.justU16 = justU16  # type: int
        self.maybeU16 = maybeU16  # type: Optional[int]
        self.defaultU16 = defaultU16  # type: int
        self.justI32 = justI32  # type: int
        self.maybeI32 = maybeI32  # type: Optional[int]
        self.defaultI32 = defaultI32  # type: int
        self.justU32 = justU32  # type: int
        self.maybeU32 = maybeU32  # type: Optional[int]
        self.defaultU32 = defaultU32  # type: int
        self.justI64 = justI64  # type: int
        self.maybeI64 = maybeI64  # type: Optional[int]
        self.defaultI64 = defaultI64  # type: int
        self.justU64 = justU64  # type: int
        self.maybeU64 = maybeU64  # type: Optional[int]
        self.defaultU64 = defaultU64  # type: int
        self.justF32 = justF32  # type: float
        self.maybeF32 = maybeF32  # type: Optional[float]
        self.defaultF32 = defaultF32  # type: float
        self.justF64 = justF64  # type: float
        self.maybeF64 = maybeF64  # type: Optional[float]
        self.defaultF64 = defaultF64  # type: float
        self.justBool = justBool  # type: bool
        self.maybeBool = maybeBool  # type: Optional[bool]
        self.defaultBool = defaultBool  # type: bool
        self.justEnum = justEnum  # type: int
        self.maybeEnum = maybeEnum  # type: Optional[int]
        self.defaultEnum = defaultEnum  # type: int

    @classmethod
    def InitFromBuf(cls, buf, pos):
        scalarStuff = ScalarStuff()
        scalarStuff.Init(buf, pos)
        return cls.InitFromObj(scalarStuff)

    @classmethod
    def InitFromPackedBuf(cls, buf, pos=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, pos)
        return cls.InitFromBuf(buf, pos+n)

    @classmethod
    def InitFromObj(cls, scalarStuff):
        x = ScalarStuffT()
        x._UnPack(scalarStuff)
        return x

    # ScalarStuffT
    def _UnPack(self, scalarStuff):
        if scalarStuff is None:
            return
        self.justI8 = scalarStuff.JustI8()
        self.maybeI8 = scalarStuff.MaybeI8()
        self.defaultI8 = scalarStuff.DefaultI8()
        self.justU8 = scalarStuff.JustU8()
        self.maybeU8 = scalarStuff.MaybeU8()
        self.defaultU8 = scalarStuff.DefaultU8()
        self.justI16 = scalarStuff.JustI16()
        self.maybeI16 = scalarStuff.MaybeI16()
        self.defaultI16 = scalarStuff.DefaultI16()
        self.justU16 = scalarStuff.JustU16()
        self.maybeU16 = scalarStuff.MaybeU16()
        self.defaultU16 = scalarStuff.DefaultU16()
        self.justI32 = scalarStuff.JustI32()
        self.maybeI32 = scalarStuff.MaybeI32()
        self.defaultI32 = scalarStuff.DefaultI32()
        self.justU32 = scalarStuff.JustU32()
        self.maybeU32 = scalarStuff.MaybeU32()
        self.defaultU32 = scalarStuff.DefaultU32()
        self.justI64 = scalarStuff.JustI64()
        self.maybeI64 = scalarStuff.MaybeI64()
        self.defaultI64 = scalarStuff.DefaultI64()
        self.justU64 = scalarStuff.JustU64()
        self.maybeU64 = scalarStuff.MaybeU64()
        self.defaultU64 = scalarStuff.DefaultU64()
        self.justF32 = scalarStuff.JustF32()
        self.maybeF32 = scalarStuff.MaybeF32()
        self.defaultF32 = scalarStuff.DefaultF32()
        self.justF64 = scalarStuff.JustF64()
        self.maybeF64 = scalarStuff.MaybeF64()
        self.defaultF64 = scalarStuff.DefaultF64()
        self.justBool = scalarStuff.JustBool()
        self.maybeBool = scalarStuff.MaybeBool()
        self.defaultBool = scalarStuff.DefaultBool()
        self.justEnum = scalarStuff.JustEnum()
        self.maybeEnum = scalarStuff.MaybeEnum()
        self.defaultEnum = scalarStuff.DefaultEnum()

    # ScalarStuffT
    def Pack(self, builder):
        ScalarStuffStart(builder)
        ScalarStuffAddJustI8(builder, self.justI8)
        ScalarStuffAddMaybeI8(builder, self.maybeI8)
        ScalarStuffAddDefaultI8(builder, self.defaultI8)
        ScalarStuffAddJustU8(builder, self.justU8)
        ScalarStuffAddMaybeU8(builder, self.maybeU8)
        ScalarStuffAddDefaultU8(builder, self.defaultU8)
        ScalarStuffAddJustI16(builder, self.justI16)
        ScalarStuffAddMaybeI16(builder, self.maybeI16)
        ScalarStuffAddDefaultI16(builder, self.defaultI16)
        ScalarStuffAddJustU16(builder, self.justU16)
        ScalarStuffAddMaybeU16(builder, self.maybeU16)
        ScalarStuffAddDefaultU16(builder, self.defaultU16)
        ScalarStuffAddJustI32(builder, self.justI32)
        ScalarStuffAddMaybeI32(builder, self.maybeI32)
        ScalarStuffAddDefaultI32(builder, self.defaultI32)
        ScalarStuffAddJustU32(builder, self.justU32)
        ScalarStuffAddMaybeU32(builder, self.maybeU32)
        ScalarStuffAddDefaultU32(builder, self.defaultU32)
        ScalarStuffAddJustI64(builder, self.justI64)
        ScalarStuffAddMaybeI64(builder, self.maybeI64)
        ScalarStuffAddDefaultI64(builder, self.defaultI64)
        ScalarStuffAddJustU64(builder, self.justU64)
        ScalarStuffAddMaybeU64(builder, self.maybeU64)
        ScalarStuffAddDefaultU64(builder, self.defaultU64)
        ScalarStuffAddJustF32(builder, self.justF32)
        ScalarStuffAddMaybeF32(builder, self.maybeF32)
        ScalarStuffAddDefaultF32(builder, self.defaultF32)
        ScalarStuffAddJustF64(builder, self.justF64)
        ScalarStuffAddMaybeF64(builder, self.maybeF64)
        ScalarStuffAddDefaultF64(builder, self.defaultF64)
        ScalarStuffAddJustBool(builder, self.justBool)
        ScalarStuffAddMaybeBool(builder, self.maybeBool)
        ScalarStuffAddDefaultBool(builder, self.defaultBool)
        ScalarStuffAddJustEnum(builder, self.justEnum)
        ScalarStuffAddMaybeEnum(builder, self.maybeEnum)
        ScalarStuffAddDefaultEnum(builder, self.defaultEnum)
        scalarStuff = ScalarStuffEnd(builder)
        return scalarStuff



###############################################################################
#
# The MIT License (MIT)
#
# Copyright (c) typedef int GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import os
import sys
import pytest
import logging

import txaio

txaio.use_twisted()

import zlmdb  # noqa

try:
    from tempfile import TemporaryDirectory
except ImportError:
    from backports.tempfile import TemporaryDirectory  # type:ignore

sys.path.append(os.path.dirname(os.path.abspath(__file__)))

if sys.version_info >= (3, 6):
    from _schema_py3 import User, Schema1, Schema3, Schema4
else:
    from _schema_py2 import User, Schema1, Schema3, Schema4


@pytest.fixture(scope="module")
def testset1():
    users = []
    for i in range(1000):
        user = User.create_test_user(i)
        users.append(user)
    return users


def test_truncate_table():
    with TemporaryDirectory() as dbpath:
        logging.info("Using temporary directory {} for database".format(dbpath))

        schema = Schema1()

        stats = zlmdb.TransactionStats()
        tabs = [
            schema.tab_oid_json,
            schema.tab_str_json,
            schema.tab_uuid_json,
            schema.tab_oid_cbor,
            schema.tab_str_cbor,
            schema.tab_uuid_cbor,
            schema.tab_oid_pickle,
            schema.tab_str_pickle,
            schema.tab_uuid_pickle,
        ]

        with zlmdb.Database(dbpath) as db:
            with db.begin(write=True, stats=stats) as txn:
                for tab in tabs:
                    tab.truncate(txn)

        logging.info(stats.puts)
        logging.info(stats.dels)


def test_fill_check(testset1):
    with TemporaryDirectory() as dbpath:
        logging.info("Using temporary directory {} for database".format(dbpath))

        schema = Schema3()

        with zlmdb.Database(dbpath) as db:
            with db.begin(write=True) as txn:
                for user in testset1:
                    schema.users[txn, user.authid] = user

        with zlmdb.Database(dbpath) as db:
            with db.begin() as txn:
                for user in testset1:
                    _user = schema.users[txn, user.authid]
                    assert _user
                    assert _user == user


def test_select(testset1):
    testset1_keys = set([user.authid for user in testset1])

    with TemporaryDirectory() as dbpath:
        logging.info("Using temporary directory {} for database".format(dbpath))

        schema = Schema3()

        with zlmdb.Database(dbpath) as db:
            with db.begin(write=True) as txn:
                for user in testset1:
                    schema.users[txn, user.authid] = user

        with zlmdb.Database(dbpath) as db:
            with db.begin() as txn:
                i = 0
                for authid, user in schema.users.select(txn):
                    i += 1
                    assert user
                    assert authid == user.authid
                    assert authid in testset1_keys


def test_count_all(testset1):
    with TemporaryDirectory() as dbpath:
        logging.info("Using temporary directory {} for database".format(dbpath))

        schema = Schema3()

        with zlmdb.Database(dbpath) as db:
            # count on empty table
            with db.begin() as txn:
                cnt = schema.users.count(txn)
                assert cnt == 0

            # fill (and count on each insert)
            with db.begin(write=True) as txn:
                i = 0
                for user in testset1:
                    schema.users[txn, user.authid] = user
                    i += 1

                    # table count within filling transaction
                    cnt = schema.users.count(txn)
                    assert cnt == i

                # table count within transaction
                cnt = schema.users.count(txn)
                assert cnt == len(testset1)

            # table count in new transaction
            with db.begin() as txn:
                cnt = schema.users.count(txn)
                assert cnt == len(testset1)

        # table count in new connection
        with zlmdb.Database(dbpath) as db:
            with db.begin() as txn:
                cnt = schema.users.count(txn)
                assert cnt == len(testset1)


def test_count_prefix(testset1):
    with TemporaryDirectory() as dbpath:
        logging.info("Using temporary directory {} for database".format(dbpath))

        schema = Schema3()

        with zlmdb.Database(dbpath) as db:
            with db.begin(write=True) as txn:
                for user in testset1:
                    schema.users[txn, user.authid] = user

        n = len(testset1)
        tests = [
            (None, n),
            ("", n),
            ("test-", n),
            ("test-1", 111),
            ("test-11", 11),
            ("test-111", 1),
        ]
        with zlmdb.Database(dbpath) as db:
            with db.begin() as txn:
                for prefix, num in tests:
                    cnt = schema.users.count(txn, prefix)
                    assert cnt == num


def test_fill_with_indexes(testset1):
    with TemporaryDirectory() as dbpath:
        logging.info("Using temporary directory {} for database".format(dbpath))

        schema = Schema4()

        with zlmdb.Database(dbpath) as db:
            stats = zlmdb.TransactionStats()

            with db.begin(write=True, stats=stats) as txn:
                for user in testset1:
                    schema.users[txn, user.oid] = user

            # check indexes has been written to (in addition to the table itself)
            num_indexes = len(schema.users.indexes())
            assert stats.puts == len(testset1) * (1 + num_indexes)


def test_truncate_table_with_index(testset1):
    with TemporaryDirectory() as dbpath:
        logging.info("Using temporary directory {} for database".format(dbpath))

        schema = Schema4()

        with zlmdb.Database(dbpath) as db:
            with db.begin(write=True) as txn:
                for user in testset1:
                    schema.users[txn, user.oid] = user

        stats = zlmdb.TransactionStats()

        with zlmdb.Database(dbpath) as db:
            with db.begin(write=True, stats=stats) as txn:
                records = schema.users.truncate(txn)
                logging.info("table truncated: {}".format(records))

        logging.info(stats.puts)
        logging.info(stats.dels)


def test_rebuild_index(testset1):
    with TemporaryDirectory() as dbpath:
        logging.info("Using temporary directory {} for database".format(dbpath))

        schema = Schema4()

        with zlmdb.Database(dbpath) as db:
            with db.begin(write=True) as txn:
                for user in testset1:
                    schema.users[txn, user.oid] = user

        with zlmdb.Database(dbpath) as db:
            with db.begin(write=True) as txn:
                records = schema.users.rebuild_index(txn, "idx1")
                logging.info(
                    '\nrebuilt specific index "idx1" on "users": {} records affected'.format(
                        records
                    )
                )


def test_rebuild_all_indexes(testset1):
    with TemporaryDirectory() as dbpath:
        logging.info("Using temporary directory {} for database".format(dbpath))

        schema = Schema4()

        with zlmdb.Database(dbpath) as db:
            with db.begin(write=True) as txn:
                for user in testset1:
                    schema.users[txn, user.oid] = user

        with zlmdb.Database(dbpath) as db:
            with db.begin(write=True) as txn:
                records = schema.users.rebuild_indexes(txn)
                logging.info(
                    '\nrebuilt all indexes on "users": {} records affected'.format(
                        records
                    )
                )



#!/usr/bin/python
# Copyright 2015 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# To run this file, use `python_sample.sh`.

# Append paths to the `flatbuffers` and `MyGame` modules. This is necessary
# to facilitate executing this script in the `samples` folder, and to root
# folder (where it gets placed when using `cmake`).
import os
import sys

sys.path.append(os.path.join(os.path.dirname(__file__), '../python'))

import flatbuffers
import MyGame.Sample.Color
import MyGame.Sample.Equipment
import MyGame.Sample.Monster
import MyGame.Sample.Vec3
import MyGame.Sample.Weapon

# Example of how to use FlatBuffers to create and read binary buffers.


def main():
  builder = flatbuffers.Builder(0)

  # Create some weapons for our Monster ('Sword' and 'Axe').
  weapon_one = builder.CreateString('Sword')
  weapon_two = builder.CreateString('Axe')

  MyGame.Sample.Weapon.WeaponStart(builder)
  MyGame.Sample.Weapon.WeaponAddName(builder, weapon_one)
  MyGame.Sample.Weapon.WeaponAddDamage(builder, 3)
  sword = MyGame.Sample.Weapon.WeaponEnd(builder)

  MyGame.Sample.Weapon.WeaponStart(builder)
  MyGame.Sample.Weapon.WeaponAddName(builder, weapon_two)
  MyGame.Sample.Weapon.WeaponAddDamage(builder, 5)
  axe = MyGame.Sample.Weapon.WeaponEnd(builder)

  # Serialize the FlatBuffer data.
  name = builder.CreateString('Orc')

  MyGame.Sample.Monster.MonsterStartInventoryVector(builder, 10)
  # Note: Since we prepend the bytes, this loop iterates in reverse order.
  for i in reversed(range(0, 10)):
    builder.PrependByte(i)
  inv = builder.EndVector()

  MyGame.Sample.Monster.MonsterStartWeaponsVector(builder, 2)
  # Note: Since we prepend the data, prepend the weapons in reverse order.
  builder.PrependUOffsetTRelative(axe)
  builder.PrependUOffsetTRelative(sword)
  weapons = builder.EndVector()

  pos = MyGame.Sample.Vec3.CreateVec3(builder, 1.0, 2.0, 3.0)

  MyGame.Sample.Monster.MonsterStart(builder)
  MyGame.Sample.Monster.MonsterAddPos(builder, pos)
  MyGame.Sample.Monster.MonsterAddHp(builder, 300)
  MyGame.Sample.Monster.MonsterAddName(builder, name)
  MyGame.Sample.Monster.MonsterAddInventory(builder, inv)
  MyGame.Sample.Monster.MonsterAddColor(
      builder, MyGame.Sample.Color.Color().Red
  )
  MyGame.Sample.Monster.MonsterAddWeapons(builder, weapons)
  MyGame.Sample.Monster.MonsterAddEquippedType(
      builder, MyGame.Sample.Equipment.Equipment().Weapon
  )
  MyGame.Sample.Monster.MonsterAddEquipped(builder, axe)
  orc = MyGame.Sample.Monster.MonsterEnd(builder)

  builder.Finish(orc)

  # We now have a FlatBuffer that we could store on disk or send over a network.

  # ...Saving to file or sending over a network code goes here...

  # Instead, we are going to access this buffer right away (as if we just
  # received it).

  buf = builder.Output()

  # Note: We use `0` for the offset here, since we got the data using the
  # `builder.Output()` method. This simulates the data you would store/receive
  # in your FlatBuffer. If you wanted to read from the `builder.Bytes` directly,
  # you would need to pass in the offset of `builder.Head()`, as the builder
  # actually constructs the buffer backwards.
  monster = MyGame.Sample.Monster.Monster.GetRootAsMonster(buf, 0)

  # Note: We did not set the `Mana` field explicitly, so we get a default value.
  assert monster.Mana() == 150
  assert monster.Hp() == 300
  assert monster.Name() == b'Orc'
  assert monster.Color() == MyGame.Sample.Color.Color().Red
  assert monster.Pos().X() == 1.0
  assert monster.Pos().Y() == 2.0
  assert monster.Pos().Z() == 3.0

  # Get and test the `inventory` FlatBuffer `vector`.
  for i in range(monster.InventoryLength()):
    assert monster.Inventory(i) == i

  # Get and test the `weapons` FlatBuffer `vector` of `table`s.
  expected_weapon_names = [b'Sword', b'Axe']
  expected_weapon_damages = [3, 5]
  for i in range(monster.WeaponsLength()):
    assert monster.Weapons(i).Name() == expected_weapon_names[i]
    assert monster.Weapons(i).Damage() == expected_weapon_damages[i]

  # Get and test the `equipped` FlatBuffer `union`.
  assert monster.EquippedType() == MyGame.Sample.Equipment.Equipment().Weapon

  # An example of how you can appropriately convert the table depending on the
  # FlatBuffer `union` type. You could add `elif` and `else` clauses to handle
  # the other FlatBuffer `union` types for this field.
  if monster.EquippedType() == MyGame.Sample.Equipment.Equipment().Weapon:
    # `monster.Equipped()` returns a `flatbuffers.Table`, which can be used
    # to initialize a `MyGame.Sample.Weapon.Weapon()`, in this case.
    union_weapon = MyGame.Sample.Weapon.Weapon()
    union_weapon.Init(monster.Equipped().Bytes, monster.Equipped().Pos)

    assert union_weapon.Name() == b'Axe'
    assert union_weapon.Damage() == 5

  print('The FlatBuffer was successfully created and verified!')


if __name__ == '__main__':
  main()



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc


class MonsterStorageStub(object):

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Store = channel.unary_unary(
        '/MyGame.Example.MonsterStorage/Store',
    )
    self.Retrieve = channel.unary_stream(
        '/MyGame.Example.MonsterStorage/Retrieve',
    )
    self.GetMaxHitPoint = channel.stream_unary(
        '/MyGame.Example.MonsterStorage/GetMaxHitPoint',
    )
    self.GetMinMaxHitPoints = channel.unary_unary(
        '/MyGame.Example.MonsterStorage/GetMinMaxHitPoints',
    )


class MonsterStorageServicer(object):

  def Store(self, request, context):
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Retrieve(self, request, context):
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetMaxHitPoint(self, request_iterator, context):
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def GetMinMaxHitPoints(self, request, context):
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_MonsterStorageServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Store': grpc.unary_unary_rpc_method_handler(
          servicer.Store,
      ),
      'Retrieve': grpc.unary_stream_rpc_method_handler(
          servicer.Retrieve,
      ),
      'GetMaxHitPoint': grpc.stream_unary_rpc_method_handler(
          servicer.GetMaxHitPoint,
      ),
      'GetMinMaxHitPoints': grpc.unary_unary_rpc_method_handler(
          servicer.GetMinMaxHitPoints,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'MyGame.Example.MonsterStorage', rpc_method_handlers
  )
  server.add_generic_rpc_handlers((generic_handler,))


try:
  # THESE ELEMENTS WILL BE DEPRECATED.
  # Please use the generated *_pb2_grpc.py files instead.
  import grpc
  from grpc.beta import implementations as beta_implementations
  from grpc.beta import interfaces as beta_interfaces
  from grpc.framework.common import cardinality
  from grpc.framework.interfaces.face import utilities as face_utilities

  class MonsterStorageStub(object):

    def __init__(self, channel):
      """Constructor.

      Args:
        channel: A grpc.Channel.
      """
      self.Store = channel.unary_unary(
          '/MyGame.Example.MonsterStorage/Store',
      )
      self.Retrieve = channel.unary_stream(
          '/MyGame.Example.MonsterStorage/Retrieve',
      )
      self.GetMaxHitPoint = channel.stream_unary(
          '/MyGame.Example.MonsterStorage/GetMaxHitPoint',
      )
      self.GetMinMaxHitPoints = channel.unary_unary(
          '/MyGame.Example.MonsterStorage/GetMinMaxHitPoints',
      )

  class MonsterStorageServicer(object):

    def Store(self, request, context):
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')

    def Retrieve(self, request, context):
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')

    def GetMaxHitPoint(self, request_iterator, context):
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')

    def GetMinMaxHitPoints(self, request, context):
      context.set_code(grpc.StatusCode.UNIMPLEMENTED)
      context.set_details('Method not implemented!')
      raise NotImplementedError('Method not implemented!')

  def add_MonsterStorageServicer_to_server(servicer, server):
    rpc_method_handlers = {
        'Store': grpc.unary_unary_rpc_method_handler(
            servicer.Store,
        ),
        'Retrieve': grpc.unary_stream_rpc_method_handler(
            servicer.Retrieve,
        ),
        'GetMaxHitPoint': grpc.stream_unary_rpc_method_handler(
            servicer.GetMaxHitPoint,
        ),
        'GetMinMaxHitPoints': grpc.unary_unary_rpc_method_handler(
            servicer.GetMinMaxHitPoints,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        'MyGame.Example.MonsterStorage', rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))

  class BetaMonsterStorageServicer(object):
    """The Beta API is deprecated for 0.15.0 and later.

    It is recommended to use the GA API (classes and functions in this
    file not marked beta) for all further purposes. This class was generated
    only to ease transition from grpcio<0.15.0 to grpcio>=0.15.0.
    """

    def Store(self, request, context):
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)

    def Retrieve(self, request, context):
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)

    def GetMaxHitPoint(self, request_iterator, context):
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)

    def GetMinMaxHitPoints(self, request, context):
      context.code(beta_interfaces.StatusCode.UNIMPLEMENTED)

  class BetaMonsterStorageStub(object):
    """The Beta API is deprecated for 0.15.0 and later.

    It is recommended to use the GA API (classes and functions in this
    file not marked beta) for all further purposes. This class was generated
    only to ease transition from grpcio<0.15.0 to grpcio>=0.15.0.
    """

    def Store(
        self,
        request,
        timeout,
        metadata=None,
        with_call=False,
        protocol_options=None,
    ):
      raise NotImplementedError()

    Store.future = None

    def Retrieve(
        self,
        request,
        timeout,
        metadata=None,
        with_call=False,
        protocol_options=None,
    ):
      raise NotImplementedError()

    def GetMaxHitPoint(
        self,
        request_iterator,
        timeout,
        metadata=None,
        with_call=False,
        protocol_options=None,
    ):
      raise NotImplementedError()

    GetMaxHitPoint.future = None

    def GetMinMaxHitPoints(
        self,
        request,
        timeout,
        metadata=None,
        with_call=False,
        protocol_options=None,
    ):
      raise NotImplementedError()

    GetMinMaxHitPoints.future = None

  def beta_create_MonsterStorage_server(
      servicer,
      pool=None,
      pool_size=None,
      default_timeout=None,
      maximum_timeout=None,
  ):
    """The Beta API is deprecated for 0.15.0 and later.

    It is recommended to use the GA API (classes and functions in this
    file not marked beta) for all further purposes. This function was
    generated only to ease transition from grpcio<0.15.0 to grpcio>=0.15.0
    """
    method_implementations = {
        (
            'MyGame.Example.MonsterStorage',
            'GetMaxHitPoint',
        ): face_utilities.stream_unary_inline(servicer.GetMaxHitPoint),
        (
            'MyGame.Example.MonsterStorage',
            'GetMinMaxHitPoints',
        ): face_utilities.unary_unary_inline(servicer.GetMinMaxHitPoints),
        (
            'MyGame.Example.MonsterStorage',
            'Retrieve',
        ): face_utilities.unary_stream_inline(servicer.Retrieve),
        (
            'MyGame.Example.MonsterStorage',
            'Store',
        ): face_utilities.unary_unary_inline(servicer.Store),
    }
    server_options = beta_implementations.server_options(
        thread_pool=pool,
        thread_pool_size=pool_size,
        default_timeout=default_timeout,
        maximum_timeout=maximum_timeout,
    )
    return beta_implementations.server(
        method_implementations, options=server_options
    )

  def beta_create_MonsterStorage_stub(
      channel, host=None, metadata_transformer=None, pool=None, pool_size=None
  ):
    """The Beta API is deprecated for 0.15.0 and later.

    It is recommended to use the GA API (classes and functions in this
    file not marked beta) for all further purposes. This function was
    generated only to ease transition from grpcio<0.15.0 to grpcio>=0.15.0
    """
    cardinalities = {
        'GetMaxHitPoint': cardinality.Cardinality.STREAM_UNARY,
        'GetMinMaxHitPoints': cardinality.Cardinality.UNARY_UNARY,
        'Retrieve': cardinality.Cardinality.UNARY_STREAM,
        'Store': cardinality.Cardinality.UNARY_UNARY,
    }
    stub_options = beta_implementations.stub_options(
        host=host,
        metadata_transformer=metadata_transformer,
        thread_pool=pool,
        thread_pool_size=pool_size,
    )
    return beta_implementations.dynamic_stub(
        channel,
        'MyGame.Example.MonsterStorage',
        cardinalities,
        options=stub_options,
    )

except ImportError:
  pass



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: crossbarfx

import flatbuffers

class Date(object):
    __slots__ = ['_tab']

    # Date
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Date
    def Year(self): return self._tab.Get(flatbuffers.number_types.Uint16Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(0))
    # Date
    def Month(self): return self._tab.Get(flatbuffers.number_types.Uint8Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(2))
    # Date
    def Day(self): return self._tab.Get(flatbuffers.number_types.Uint8Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(3))

def CreateDate(builder, year, month, day):
    builder.Prep(2, 4)
    builder.PrependUint8(day)
    builder.PrependUint8(month)
    builder.PrependUint16(year)
    return builder.Offset()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: Example

# Composite components of Monster color.
class Color(object):
    Red = 1
    # \brief color Green
    # Green is bit_flag with value (1u << 1)
    Green = 2
    # \brief color Blue (1u << 3)
    Blue = 8



###############################################################################
#
# The MIT License (MIT)
#
# Copyright (c) typedef int GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
###############################################################################

import random
import uuid
import datetime
from typing import Optional, List, Dict

import zlmdb


class User(object):
    oid: int
    name: str
    authid: str
    uuid: uuid.UUID
    email: str
    birthday: datetime.date
    is_friendly: bool
    tags: Optional[List[str]]
    ratings: Dict[str, float]
    friends: List[int]
    referred_by: int
    realm_oid: int
    icecream: str
    mrealm: uuid.UUID  # type:ignore
    mrealm_notnull: uuid.UUID  # type:ignore

    def __init__(self):
        self.oid = None
        self.name = None
        self.authid = None
        self.uuid = None
        self.email = None
        self.birthday = None
        self.is_friendly = None
        self.tags = None
        self.ratings = {}
        self.friends = []
        self.referred_by = None
        self.realm_oid = None
        self.icecream = None
        self.mrealm = None
        self.mrealm_notnull = None

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return False
        if other.oid != self.oid:
            return False
        if other.name != self.name:
            return False
        if other.authid != self.authid:
            return False
        if other.uuid != self.uuid:
            return False
        if other.email != self.email:
            return False
        if other.birthday != self.birthday:
            return False
        if other.is_friendly != self.is_friendly:
            return False
        if (self.tags and not other.tags) or (not self.tags and other.tags):
            return False
        if other.realm_oid != self.realm_oid:
            return False
        if other.icecream != self.icecream:
            return False
        if other.mrealm != self.mrealm:
            return False
        if other.mrealm_notnull != self.mrealm_notnull:
            return False
        return True

    def __ne__(self, other):
        return not self.__eq__(other)

    def marshal(self):
        obj = {
            "oid": self.oid,
            "name": self.name,
            "authid": self.authid,
            "uuid": self.uuid.hex if self.uuid else None,
            "email": self.email,
            "birthday": {
                "year": self.birthday.year if self.birthday else None,
                "month": self.birthday.month if self.birthday else None,
                "day": self.birthday.day if self.birthday else None,
            },
            "is_friendly": self.is_friendly,
            "tags": self.tags,
            "ratings": self.ratings,
            "friends": self.friends,
            "referred_by": self.referred_by,
            "realm_oid": self.realm_oid,
            "icecream": self.icecream,
            "mrealm": self.mrealm.hex if self.mrealm else None,
            "mrealm_notnull": self.mrealm_notnull.hex if self.mrealm_notnull else None,
        }
        return obj

    @staticmethod
    def parse(obj):
        user = User()
        user.oid = obj.get("oid", None)
        user.name = obj.get("name", None)
        user.authid = obj.get("authid", None)
        if "uuid" in obj:
            user.uuid = uuid.UUID(hex=obj["uuid"])
        user.email = obj.get("email", None)
        if "birthday" in obj:
            b = obj["birthday"]
            user.birthday = datetime.date(b.year, b.month, b.day)
        user.is_friendly = obj.get("is_friendly", None)
        user.tags = obj.get("tags", None)
        user.ratings = obj.get("ratings", {})
        user.friends = obj.get("friends", [])
        user.referred_by = obj.get("referred_by", None)
        user.realm_oid = obj.get("realm_oid", None)
        user.icecream = obj.get("icecream", None)
        if "mrealm" in obj and obj["mrealm"]:
            user.mrealm = uuid.UUID(hex=obj["mrealm"])
        if "mrealm_notnull" in obj and obj["mrealm_notnull"]:
            user.mrealm_notnull = uuid.UUID(hex=obj["mrealm_notnull"])
        return user

    @staticmethod
    def create_test_user(oid=None, realm_oid=None):
        user = User()
        if oid is not None:
            user.oid = oid
        else:
            user.oid = random.randint(0, 9007199254740992)
        user.name = "Test {}".format(user.oid)
        user.authid = "test-{}".format(user.oid)
        user.uuid = uuid.uuid4()
        user.email = "{}@example.com".format(user.authid)
        user.birthday = datetime.date(1950, 12, 24)
        user.is_friendly = True
        user.tags = ["geek", "sudoko", "yellow"]
        for j in range(10):
            user.ratings["test-rating-{}".format(j)] = 1 / (
                j + 1
            )  # round(random.random(), 3)
        user.friends = [random.randint(0, 9007199254740992) for _ in range(10)]
        user.referred_by = random.randint(0, 9007199254740992)
        if realm_oid is not None:
            user.realm_oid = realm_oid
        else:
            user.realm_oid = random.randint(0, 9007199254740992)
        user.icecream = random.choice(["vanilla", "lemon", "strawberry"])
        user.mrealm = uuid.uuid4()
        user.mrealm_notnull = uuid.uuid4()
        return user


class Schema1(zlmdb.Schema):
    tab_uuid_str: zlmdb.MapUuidString
    tab_uuid_oid: zlmdb.MapUuidOid
    tab_uuid_uuid: zlmdb.MapUuidUuid
    tab_str_str: zlmdb.MapStringString
    tab_str_oid: zlmdb.MapStringOid
    tab_str_uuid: zlmdb.MapStringUuid
    tab_oid_str: zlmdb.MapOidString
    tab_oid_oid: zlmdb.MapOidOid
    tab_oid_uuid: zlmdb.MapOidUuid
    tab_uuid_json: zlmdb.MapUuidJson
    tab_uuid_cbor: zlmdb.MapUuidCbor
    tab_uuid_pickle: zlmdb.MapUuidPickle
    tab_str_json: zlmdb.MapStringJson
    tab_str_cbor: zlmdb.MapStringCbor
    tab_str_pickle: zlmdb.MapStringPickle
    tab_oid_json: zlmdb.MapOidJson
    tab_oid_cbor: zlmdb.MapOidCbor
    tab_oid_pickle: zlmdb.MapOidPickle

    def __init__(self):
        self.tab_uuid_str = zlmdb.MapUuidString(slot=1)
        self.tab_uuid_oid = zlmdb.MapUuidOid(slot=2)
        self.tab_uuid_uuid = zlmdb.MapUuidUuid(slot=3)
        self.tab_str_str = zlmdb.MapStringString(slot=4)
        self.tab_str_oid = zlmdb.MapStringOid(slot=5)
        self.tab_str_uuid = zlmdb.MapStringUuid(slot=6)
        self.tab_oid_str = zlmdb.MapOidString(slot=7)
        self.tab_oid_oid = zlmdb.MapOidOid(slot=8)
        self.tab_oid_uuid = zlmdb.MapOidUuid(slot=9)
        self.tab_uuid_json = zlmdb.MapUuidJson(
            slot=10, marshal=(lambda o: o.marshal()), unmarshal=User.parse
        )
        self.tab_uuid_cbor = zlmdb.MapUuidCbor(
            slot=11, marshal=(lambda o: o.marshal()), unmarshal=User.parse
        )
        self.tab_uuid_pickle = zlmdb.MapUuidPickle(slot=12)
        self.tab_str_json = zlmdb.MapStringJson(
            slot=20, marshal=(lambda o: o.marshal()), unmarshal=User.parse
        )
        self.tab_str_cbor = zlmdb.MapStringCbor(
            slot=21, marshal=(lambda o: o.marshal()), unmarshal=User.parse
        )
        self.tab_str_pickle = zlmdb.MapStringPickle(slot=22)
        self.tab_oid_json = zlmdb.MapOidJson(
            slot=30, marshal=(lambda o: o.marshal()), unmarshal=User.parse
        )
        self.tab_oid_cbor = zlmdb.MapOidCbor(
            slot=31, marshal=(lambda o: o.marshal()), unmarshal=User.parse
        )
        self.tab_oid_pickle = zlmdb.MapOidPickle(slot=32)


class Schema2(zlmdb.Schema):
    users: zlmdb.MapOidPickle

    def __init__(self):
        self.users = zlmdb.MapOidPickle(1)


class Schema3(zlmdb.Schema):
    users: zlmdb.MapStringPickle

    def __init__(self):
        self.users = zlmdb.MapStringPickle(1)


class Schema4(zlmdb.Schema):
    users: zlmdb.MapOidPickle

    idx_users_by_authid: zlmdb.MapStringOid

    idx_users_by_email: zlmdb.MapStringOid

    idx_users_by_realm: zlmdb.MapOidOidOid

    idx_users_by_icecream: zlmdb.MapStringOidOid

    idx_users_by_mrealm_authid: zlmdb.MapUuidStringOid

    idx_users_by_mrealm_authid_notnull: zlmdb.MapUuidStringOid

    def __init__(self):
        super(Schema4, self).__init__()

        self.users = zlmdb.MapOidPickle(1)

        self.idx_users_by_authid = zlmdb.MapStringOid(2)
        self.users.attach_index(
            "idx1", self.idx_users_by_authid, lambda user: user.authid, nullable=False
        )

        self.idx_users_by_email = zlmdb.MapStringOid(3)
        self.users.attach_index(
            "idx2", self.idx_users_by_email, lambda user: user.email, nullable=True
        )

        self.idx_users_by_realm = zlmdb.MapOidOidOid(4)
        self.users.attach_index(
            "idx3",
            self.idx_users_by_realm,
            lambda user: (user.realm_oid, user.oid),
            nullable=False,
        )

        self.idx_users_by_icecream = zlmdb.MapStringOidOid(5)
        self.users.attach_index(
            "idx4",
            self.idx_users_by_icecream,
            lambda user: (user.icecream, user.oid),
            nullable=False,
        )

        self.idx_users_by_mrealm_authid = zlmdb.MapUuidStringOid(6)
        self.users.attach_index(
            "idx5",
            self.idx_users_by_mrealm_authid,
            lambda user: (user.mrealm, user.authid),
            nullable=True,
        )

        self.idx_users_by_mrealm_notnull_authid = zlmdb.MapUuidStringOid(7)
        self.users.attach_index(
            "idx6",
            self.idx_users_by_mrealm_notnull_authid,
            lambda user: (user.mrealm_notnull, user.authid),
            nullable=False,
        )



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: reflection

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class Field(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = Field()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsField(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    @classmethod
    def FieldBufferHasIdentifier(cls, buf, offset, size_prefixed=False):
        return flatbuffers.util.BufferHasIdentifier(buf, offset, b"\x42\x46\x42\x53", size_prefixed=size_prefixed)

    # Field
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Field
    def Name(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

    # Field
    def Type(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
        if o != 0:
            x = self._tab.Indirect(o + self._tab.Pos)
            from reflection.Type import Type
            obj = Type()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Field
    def Id(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(8))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint16Flags, o + self._tab.Pos)
        return 0

    # Field
    def Offset(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(10))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint16Flags, o + self._tab.Pos)
        return 0

    # Field
    def DefaultInteger(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(12))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Int64Flags, o + self._tab.Pos)
        return 0

    # Field
    def DefaultReal(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(14))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Float64Flags, o + self._tab.Pos)
        return 0.0

    # Field
    def Deprecated(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(16))
        if o != 0:
            return bool(self._tab.Get(flatbuffers.number_types.BoolFlags, o + self._tab.Pos))
        return False

    # Field
    def Required(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(18))
        if o != 0:
            return bool(self._tab.Get(flatbuffers.number_types.BoolFlags, o + self._tab.Pos))
        return False

    # Field
    def Key(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(20))
        if o != 0:
            return bool(self._tab.Get(flatbuffers.number_types.BoolFlags, o + self._tab.Pos))
        return False

    # Field
    def Attributes(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(22))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from reflection.KeyValue import KeyValue
            obj = KeyValue()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # Field
    def AttributesLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(22))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Field
    def AttributesIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(22))
        return o == 0

    # Field
    def Documentation(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(24))
        if o != 0:
            a = self._tab.Vector(o)
            return self._tab.String(a + flatbuffers.number_types.UOffsetTFlags.py_type(j * 4))
        return ""

    # Field
    def DocumentationLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(24))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # Field
    def DocumentationIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(24))
        return o == 0

    # Field
    def Optional(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(26))
        if o != 0:
            return bool(self._tab.Get(flatbuffers.number_types.BoolFlags, o + self._tab.Pos))
        return False

    # Number of padding octets to always add after this field. Structs only.
    # Field
    def Padding(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(28))
        if o != 0:
            return self._tab.Get(flatbuffers.number_types.Uint16Flags, o + self._tab.Pos)
        return 0

def FieldStart(builder): builder.StartObject(13)
def Start(builder):
    return FieldStart(builder)
def FieldAddName(builder, name): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(name), 0)
def AddName(builder, name):
    return FieldAddName(builder, name)
def FieldAddType(builder, type): builder.PrependUOffsetTRelativeSlot(1, flatbuffers.number_types.UOffsetTFlags.py_type(type), 0)
def AddType(builder, type):
    return FieldAddType(builder, type)
def FieldAddId(builder, id): builder.PrependUint16Slot(2, id, 0)
def AddId(builder, id):
    return FieldAddId(builder, id)
def FieldAddOffset(builder, offset): builder.PrependUint16Slot(3, offset, 0)
def AddOffset(builder, offset):
    return FieldAddOffset(builder, offset)
def FieldAddDefaultInteger(builder, defaultInteger): builder.PrependInt64Slot(4, defaultInteger, 0)
def AddDefaultInteger(builder, defaultInteger):
    return FieldAddDefaultInteger(builder, defaultInteger)
def FieldAddDefaultReal(builder, defaultReal): builder.PrependFloat64Slot(5, defaultReal, 0.0)
def AddDefaultReal(builder, defaultReal):
    return FieldAddDefaultReal(builder, defaultReal)
def FieldAddDeprecated(builder, deprecated): builder.PrependBoolSlot(6, deprecated, 0)
def AddDeprecated(builder, deprecated):
    return FieldAddDeprecated(builder, deprecated)
def FieldAddRequired(builder, required): builder.PrependBoolSlot(7, required, 0)
def AddRequired(builder, required):
    return FieldAddRequired(builder, required)
def FieldAddKey(builder, key): builder.PrependBoolSlot(8, key, 0)
def AddKey(builder, key):
    return FieldAddKey(builder, key)
def FieldAddAttributes(builder, attributes): builder.PrependUOffsetTRelativeSlot(9, flatbuffers.number_types.UOffsetTFlags.py_type(attributes), 0)
def AddAttributes(builder, attributes):
    return FieldAddAttributes(builder, attributes)
def FieldStartAttributesVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartAttributesVector(builder, numElems):
    return FieldStartAttributesVector(builder, numElems)
def FieldAddDocumentation(builder, documentation): builder.PrependUOffsetTRelativeSlot(10, flatbuffers.number_types.UOffsetTFlags.py_type(documentation), 0)
def AddDocumentation(builder, documentation):
    return FieldAddDocumentation(builder, documentation)
def FieldStartDocumentationVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def StartDocumentationVector(builder, numElems):
    return FieldStartDocumentationVector(builder, numElems)
def FieldAddOptional(builder, optional): builder.PrependBoolSlot(11, optional, 0)
def AddOptional(builder, optional):
    return FieldAddOptional(builder, optional)
def FieldAddPadding(builder, padding): builder.PrependUint16Slot(12, padding, 0)
def AddPadding(builder, padding):
    return FieldAddPadding(builder, padding)
def FieldEnd(builder): return builder.EndObject()
def End(builder):
    return FieldEnd(builder)


#!/usr/bin/env python3
#
# Copyright 2021 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
import platform
import subprocess

# Get the path where this script is located so we can invoke the script from
# any directory and have the paths work correctly.
script_path = Path(__file__).parent.resolve()

# Get the root path as an absolute path, so all derived paths are absolute.
root_path = script_path.parent.parent.absolute()

# Get the location of the flatc executable, reading from the first command line
# argument or defaulting to default names.
flatc_exe = Path(
    ("flatc" if not platform.system() == "Windows" else "flatc.exe")
)

# Find and assert flatc compiler is present.
if root_path in flatc_exe.parents:
  flatc_exe = flatc_exe.relative_to(root_path)
flatc_path = Path(root_path, flatc_exe)
assert flatc_path.exists(), "Cannot find the flatc compiler " + str(flatc_path)

# Specify the other paths that will be referenced
tests_path = Path(script_path, "tests")


def flatc_annotate(schema, file, cwd=script_path):
  cmd = [str(flatc_path), "--annotate", schema, file]
  result = subprocess.run(cmd, cwd=str(cwd), check=True)


test_files = [
    "annotated_binary.bin",
    "tests/invalid_root_offset.bin",
    "tests/invalid_root_table_too_short.bin",
    "tests/invalid_root_table_vtable_offset.bin",
    "tests/invalid_string_length.bin",
    "tests/invalid_string_length_cut_short.bin",
    "tests/invalid_struct_array_field_cut_short.bin",
    "tests/invalid_struct_field_cut_short.bin",
    "tests/invalid_table_field_size.bin",
    "tests/invalid_table_field_offset.bin",
    "tests/invalid_union_type_value.bin",
    "tests/invalid_vector_length_cut_short.bin",
    "tests/invalid_vector_scalars_cut_short.bin",
    "tests/invalid_vector_strings_cut_short.bin",
    "tests/invalid_vector_structs_cut_short.bin",
    "tests/invalid_vector_tables_cut_short.bin",
    "tests/invalid_vector_unions_cut_short.bin",
    "tests/invalid_vector_union_type_value.bin",
    "tests/invalid_vtable_ref_table_size_short.bin",
    "tests/invalid_vtable_ref_table_size.bin",
    "tests/invalid_vtable_size_short.bin",
    "tests/invalid_vtable_size.bin",
    "tests/invalid_vtable_field_offset.bin",
]

for test_file in test_files:
  flatc_annotate("annotated_binary.fbs", test_file)



#!/usr/bin/env python3
#
# Copyright 2022 Google Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from pathlib import Path
from util import flatc, root_path

grpc_examples_path = Path(root_path, "grpc/examples")

greeter_schema = str(Path(grpc_examples_path, "greeter.fbs"))

COMMON_ARGS = [
    "--grpc",
    "--bfbs-filenames",
    str(grpc_examples_path),
]


def GenerateGRPCExamples():

  flatc(
      COMMON_ARGS
      + [
          "--go",
      ],
      schema=greeter_schema,
      cwd=Path(grpc_examples_path, "go/greeter"),
  )

  flatc(
      COMMON_ARGS
      + [
          "--python",
      ],
      schema=greeter_schema,
      cwd=Path(grpc_examples_path, "python/greeter"),
  )

  flatc(
      COMMON_ARGS
      + [
          "--swift",
          "--gen-json-emit",
      ],
      schema=greeter_schema,
      cwd=Path(grpc_examples_path, "swift/Greeter/Sources/Model"),
  )

  flatc(
      COMMON_ARGS
      + [
          "--ts",
      ],
      schema=greeter_schema,
      cwd=Path(grpc_examples_path, "ts/greeter/src"),
  )


if __name__ == "__main__":
  GenerateGRPCExamples()



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: NamespaceB

import flatbuffers
from flatbuffers.compat import import_numpy

np = import_numpy()


class TableInNestedNS(object):
  __slots__ = ['_tab']

  @classmethod
  def GetRootAs(cls, buf, offset=0):
    n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
    x = TableInNestedNS()
    x.Init(buf, n + offset)
    return x

  @classmethod
  def GetRootAsTableInNestedNS(cls, buf, offset=0):
    """This method is deprecated. Please switch to GetRootAs."""
    return cls.GetRootAs(buf, offset)

  # TableInNestedNS
  def Init(self, buf, pos):
    self._tab = flatbuffers.table.Table(buf, pos)

  # TableInNestedNS
  def Foo(self):
    o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
    if o != 0:
      return self._tab.Get(
          flatbuffers.number_types.Int32Flags, o + self._tab.Pos
      )
    return 0


def TableInNestedNSStart(builder):
  builder.StartObject(1)


def Start(builder):
  return TableInNestedNSStart(builder)


def TableInNestedNSAddFoo(builder, foo):
  builder.PrependInt32Slot(0, foo, 0)


def AddFoo(builder, foo):
  return TableInNestedNSAddFoo(builder, foo)


def TableInNestedNSEnd(builder):
  return builder.EndObject()


def End(builder):
  return TableInNestedNSEnd(builder)


class TableInNestedNST(object):

  # TableInNestedNST
  def __init__(self):
    self.foo = 0  # type: int

  @classmethod
  def InitFromBuf(cls, buf, pos):
    tableInNestedNS = TableInNestedNS()
    tableInNestedNS.Init(buf, pos)
    return cls.InitFromObj(tableInNestedNS)

  @classmethod
  def InitFromObj(cls, tableInNestedNS):
    x = TableInNestedNST()
    x._UnPack(tableInNestedNS)
    return x

  # TableInNestedNST
  def _UnPack(self, tableInNestedNS):
    if tableInNestedNS is None:
      return
    self.foo = tableInNestedNS.Foo()

  # TableInNestedNST
  def Pack(self, builder):
    TableInNestedNSStart(builder)
    TableInNestedNSAddFoo(builder, self.foo)
    tableInNestedNS = TableInNestedNSEnd(builder)
    return tableInNestedNS



from __future__ import annotations

import flatbuffers
import numpy as np

import typing

uoffset: typing.TypeAlias = flatbuffers.number_types.UOffsetTFlags.py_type

class MonsterExtra(object):
  @classmethod
  def GetRootAs(cls, buf: bytes, offset: int) -> MonsterExtra: ...
  @classmethod
  def GetRootAsMonsterExtra(cls, buf: bytes, offset: int) -> MonsterExtra: ...
  @classmethod
  def MonsterExtraBufferHasIdentifier(cls, buf: bytes, offset: int, size_prefixed: bool) -> bool: ...
  def Init(self, buf: bytes, pos: int) -> None: ...
  def D0(self) -> float: ...
  def D1(self) -> float: ...
  def D2(self) -> float: ...
  def D3(self) -> float: ...
  def F0(self) -> float: ...
  def F1(self) -> float: ...
  def F2(self) -> float: ...
  def F3(self) -> float: ...
  def Dvec(self, i: int) -> typing.List[float]: ...
  def DvecAsNumpy(self) -> np.ndarray: ...
  def DvecLength(self) -> int: ...
  def DvecIsNone(self) -> bool: ...
  def Fvec(self, i: int) -> typing.List[float]: ...
  def FvecAsNumpy(self) -> np.ndarray: ...
  def FvecLength(self) -> int: ...
  def FvecIsNone(self) -> bool: ...
class MonsterExtraT(object):
  d0: float
  d1: float
  d2: float
  d3: float
  f0: float
  f1: float
  f2: float
  f3: float
  dvec: typing.List[float]
  fvec: typing.List[float]
  def __init__(
    self,
    d0: float = ...,
    d1: float = ...,
    d2: float = ...,
    d3: float = ...,
    f0: float = ...,
    f1: float = ...,
    f2: float = ...,
    f3: float = ...,
    dvec: typing.List[float] | None = ...,
    fvec: typing.List[float] | None = ...,
  ) -> None: ...
  @classmethod
  def InitFromBuf(cls, buf: bytes, pos: int) -> MonsterExtraT: ...
  @classmethod
  def InitFromPackedBuf(cls, buf: bytes, pos: int = 0) -> MonsterExtraT: ...
  @classmethod
  def InitFromObj(cls, monsterExtra: MonsterExtra) -> MonsterExtraT: ...
  def _UnPack(self, monsterExtra: MonsterExtra) -> None: ...
  def Pack(self, builder: flatbuffers.Builder) -> None: ...
  def __eq__(self, other: MonsterExtraT) -> bool: ...
def MonsterExtraStart(builder: flatbuffers.Builder) -> None: ...
def Start(builder: flatbuffers.Builder) -> None: ...
def MonsterExtraAddD0(builder: flatbuffers.Builder, d0: float) -> None: ...
def MonsterExtraAddD1(builder: flatbuffers.Builder, d1: float) -> None: ...
def MonsterExtraAddD2(builder: flatbuffers.Builder, d2: float) -> None: ...
def MonsterExtraAddD3(builder: flatbuffers.Builder, d3: float) -> None: ...
def MonsterExtraAddF0(builder: flatbuffers.Builder, f0: float) -> None: ...
def MonsterExtraAddF1(builder: flatbuffers.Builder, f1: float) -> None: ...
def MonsterExtraAddF2(builder: flatbuffers.Builder, f2: float) -> None: ...
def MonsterExtraAddF3(builder: flatbuffers.Builder, f3: float) -> None: ...
def MonsterExtraAddDvec(builder: flatbuffers.Builder, dvec: uoffset) -> None: ...
def MonsterExtraStartDvecVector(builder: flatbuffers.Builder, num_elems: int) -> uoffset: ...
def StartDvecVector(builder: flatbuffers.Builder, num_elems: int) -> uoffset: ...
def MonsterExtraAddFvec(builder: flatbuffers.Builder, fvec: uoffset) -> None: ...
def MonsterExtraStartFvecVector(builder: flatbuffers.Builder, num_elems: int) -> uoffset: ...
def StartFvecVector(builder: flatbuffers.Builder, num_elems: int) -> uoffset: ...
def MonsterExtraEnd(builder: flatbuffers.Builder) -> uoffset: ...
def End(builder: flatbuffers.Builder) -> uoffset: ...




import uuid
import datetime
from typing import Optional, List, Dict


class User(object):
    oid: int
    name: str
    authid: str
    uuid: uuid.UUID
    email: str
    birthday: datetime.date
    is_friendly: bool
    tags: Optional[List[str]]
    ratings: Dict[str, float] = {}
    friends: List[int] = []
    referred_by: int = None

    def marshal(self):
        obj = {
            'oid': self.oid,
            'name': self.name,
            'authid': self.authid,
            'uuid': self.uuid.hex if self.uuid else None,
            'email': self.email,
            'birthday': {
                'year': self.birthday.year if self.birthday else None,
                'month': self.birthday.month if self.birthday else None,
                'day': self.birthday.day if self.birthday else None,
            },
            'is_friendly': self.is_friendly,
            'tags': self.tags,
            'ratings': self.ratings,
            'friends': self.friends,
            'referred_by': self.referred_by,
        }
        return obj

    @staticmethod
    def parse(obj):
        user = User()
        user.oid = obj.get('oid', None)
        user.name = obj.get('name', None)
        user.authid = obj.get('authid', None)
        if 'uuid' in obj:
            user.uuid = uuid.UUID(hex=obj['uuid'])
        user.email = obj.get('email', None)
        if 'birthday' in obj:
            b = obj['birthday']
            user.birthday = datetime.date(b.year, b.month, b.day)
        user.is_friendly = obj.get('is_friendly', None)
        user.tags = obj.get('tags', None)
        user.ratings = obj.get('ratings', {})
        user.friends = obj.get('friends', [])
        user.referred_by = obj.get('referred_by', None)
        return user



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: Example

import flatbuffers
from flatbuffers.compat import import_numpy
from typing import Any
from MyGame.Example.ArrayStruct import ArrayStruct
from typing import Optional
np = import_numpy()

class ArrayTable(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAs(cls, buf, offset: int = 0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = ArrayTable()
        x.Init(buf, n + offset)
        return x

    @classmethod
    def GetRootAsArrayTable(cls, buf, offset=0):
        """This method is deprecated. Please switch to GetRootAs."""
        return cls.GetRootAs(buf, offset)
    @classmethod
    def ArrayTableBufferHasIdentifier(cls, buf, offset, size_prefixed=False):
        return flatbuffers.util.BufferHasIdentifier(buf, offset, b"\x41\x52\x52\x54", size_prefixed=size_prefixed)

    # ArrayTable
    def Init(self, buf: bytes, pos: int):
        self._tab = flatbuffers.table.Table(buf, pos)

    # ArrayTable
    def A(self) -> Optional[ArrayStruct]:
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = o + self._tab.Pos
            obj = ArrayStruct()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

def ArrayTableStart(builder: flatbuffers.Builder):
    builder.StartObject(1)

def Start(builder: flatbuffers.Builder):
    ArrayTableStart(builder)

def ArrayTableAddA(builder: flatbuffers.Builder, a: Any):
    builder.PrependStructSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(a), 0)

def AddA(builder: flatbuffers.Builder, a: Any):
    ArrayTableAddA(builder, a)

def ArrayTableEnd(builder: flatbuffers.Builder) -> int:
    return builder.EndObject()

def End(builder: flatbuffers.Builder) -> int:
    return ArrayTableEnd(builder)

import MyGame.Example.ArrayStruct
try:
    from typing import Optional
except:
    pass

class ArrayTableT(object):

    # ArrayTableT
    def __init__(
        self,
        a = None,
    ):
        self.a = a  # type: Optional[MyGame.Example.ArrayStruct.ArrayStructT]

    @classmethod
    def InitFromBuf(cls, buf, pos):
        arrayTable = ArrayTable()
        arrayTable.Init(buf, pos)
        return cls.InitFromObj(arrayTable)

    @classmethod
    def InitFromPackedBuf(cls, buf, pos=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, pos)
        return cls.InitFromBuf(buf, pos+n)

    @classmethod
    def InitFromObj(cls, arrayTable):
        x = ArrayTableT()
        x._UnPack(arrayTable)
        return x

    # ArrayTableT
    def _UnPack(self, arrayTable):
        if arrayTable is None:
            return
        if arrayTable.A() is not None:
            self.a = MyGame.Example.ArrayStruct.ArrayStructT.InitFromObj(arrayTable.A())

    # ArrayTableT
    def Pack(self, builder):
        ArrayTableStart(builder)
        if self.a is not None:
            a = self.a.Pack(builder)
            ArrayTableAddA(builder, a)
        arrayTable = ArrayTableEnd(builder)
        return arrayTable



from golden_utils import flatc_golden


def flatc(options, schema):
  # Wrap the golden flatc generator with Rust specifics
  flatc_golden(options=["--rust"] + options, schema=schema, prefix="rust")


def GenerateRust():
  flatc([], "basic.fbs")



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: Example

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class Ability(object):
    __slots__ = ['_tab']

    @classmethod
    def SizeOf(cls):
        return 8

    # Ability
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # Ability
    def Id(self): return self._tab.Get(flatbuffers.number_types.Uint32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(0))
    # Ability
    def Distance(self): return self._tab.Get(flatbuffers.number_types.Uint32Flags, self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(4))

def CreateAbility(builder, id, distance):
    builder.Prep(4, 8)
    builder.PrependUint32(distance)
    builder.PrependUint32(id)
    return builder.Offset()


class AbilityT(object):

    # AbilityT
    def __init__(
        self,
        id = 0,
        distance = 0,
    ):
        self.id = id  # type: int
        self.distance = distance  # type: int

    @classmethod
    def InitFromBuf(cls, buf, pos):
        ability = Ability()
        ability.Init(buf, pos)
        return cls.InitFromObj(ability)

    @classmethod
    def InitFromPackedBuf(cls, buf, pos=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, pos)
        return cls.InitFromBuf(buf, pos+n)

    @classmethod
    def InitFromObj(cls, ability):
        x = AbilityT()
        x._UnPack(ability)
        return x

    # AbilityT
    def _UnPack(self, ability):
        if ability is None:
            return
        self.id = ability.Id()
        self.distance = ability.Distance()

    # AbilityT
    def Pack(self, builder):
        return CreateAbility(builder, self.id, self.distance)



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: reflection


class BaseType(object):
    None_ = 0
    UType = 1
    Bool = 2
    Byte = 3
    UByte = 4
    Short = 5
    UShort = 6
    Int = 7
    UInt = 8
    Long = 9
    ULong = 10
    Float = 11
    Double = 12
    String = 13
    Vector = 14
    Obj = 15
    Union = 16
    Array = 17
    MaxBaseType = 18



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: NamespaceB

import flatbuffers
from flatbuffers.compat import import_numpy

np = import_numpy()


class StructInNestedNS(object):
  __slots__ = ['_tab']

  @classmethod
  def SizeOf(cls):
    return 8

  # StructInNestedNS
  def Init(self, buf, pos):
    self._tab = flatbuffers.table.Table(buf, pos)

  # StructInNestedNS
  def A(self):
    return self._tab.Get(
        flatbuffers.number_types.Int32Flags,
        self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(0),
    )

  # StructInNestedNS
  def B(self):
    return self._tab.Get(
        flatbuffers.number_types.Int32Flags,
        self._tab.Pos + flatbuffers.number_types.UOffsetTFlags.py_type(4),
    )


def CreateStructInNestedNS(builder, a, b):
  builder.Prep(4, 8)
  builder.PrependInt32(b)
  builder.PrependInt32(a)
  return builder.Offset()


class StructInNestedNST(object):

  # StructInNestedNST
  def __init__(self):
    self.a = 0  # type: int
    self.b = 0  # type: int

  @classmethod
  def InitFromBuf(cls, buf, pos):
    structInNestedNS = StructInNestedNS()
    structInNestedNS.Init(buf, pos)
    return cls.InitFromObj(structInNestedNS)

  @classmethod
  def InitFromObj(cls, structInNestedNS):
    x = StructInNestedNST()
    x._UnPack(structInNestedNS)
    return x

  # StructInNestedNST
  def _UnPack(self, structInNestedNS):
    if structInNestedNS is None:
      return
    self.a = structInNestedNS.A()
    self.b = structInNestedNS.B()

  # StructInNestedNST
  def Pack(self, builder):
    return CreateStructInNestedNS(builder, self.a, self.b)



from __future__ import annotations

import flatbuffers
import numpy as np

import typing
from MyGame.Example.NestedUnion.Any import Any
from MyGame.Example.NestedUnion.TestSimpleTableWithEnum import TestSimpleTableWithEnumT
from MyGame.Example.NestedUnion.Vec3 import Vec3T
from flatbuffers import table

uoffset: typing.TypeAlias = flatbuffers.number_types.UOffsetTFlags.py_type

class NestedUnionTest(object):
  @classmethod
  def GetRootAs(cls, buf: bytes, offset: int) -> NestedUnionTest: ...
  @classmethod
  def GetRootAsNestedUnionTest(cls, buf: bytes, offset: int) -> NestedUnionTest: ...
  def Init(self, buf: bytes, pos: int) -> None: ...
  def Name(self) -> str | None: ...
  def DataType(self) -> typing.Literal[Any.NONE, Any.Vec3, Any.TestSimpleTableWithEnum]: ...
  def Data(self) -> table.Table | None: ...
  def Id(self) -> int: ...
class NestedUnionTestT(object):
  name: str | None
  dataType: typing.Literal[Any.NONE, Any.Vec3, Any.TestSimpleTableWithEnum]
  data: typing.Union[None, Vec3T, TestSimpleTableWithEnumT]
  id: int
  def __init__(
    self,
    name: str | None = ...,
    dataType: typing.Literal[Any.NONE, Any.Vec3, Any.TestSimpleTableWithEnum] = ...,
    data: typing.Union[None, Vec3T, TestSimpleTableWithEnumT] = ...,
    id: int = ...,
  ) -> None: ...
  @classmethod
  def InitFromBuf(cls, buf: bytes, pos: int) -> NestedUnionTestT: ...
  @classmethod
  def InitFromPackedBuf(cls, buf: bytes, pos: int = 0) -> NestedUnionTestT: ...
  @classmethod
  def InitFromObj(cls, nestedUnionTest: NestedUnionTest) -> NestedUnionTestT: ...
  def _UnPack(self, nestedUnionTest: NestedUnionTest) -> None: ...
  def Pack(self, builder: flatbuffers.Builder) -> None: ...
def NestedUnionTestStart(builder: flatbuffers.Builder) -> None: ...
def Start(builder: flatbuffers.Builder) -> None: ...
def NestedUnionTestAddName(builder: flatbuffers.Builder, name: uoffset) -> None: ...
def NestedUnionTestAddDataType(builder: flatbuffers.Builder, dataType: typing.Literal[Any.NONE, Any.Vec3, Any.TestSimpleTableWithEnum]) -> None: ...
def NestedUnionTestAddData(builder: flatbuffers.Builder, data: uoffset) -> None: ...
def NestedUnionTestAddId(builder: flatbuffers.Builder, id: int) -> None: ...
def NestedUnionTestEnd(builder: flatbuffers.Builder) -> uoffset: ...
def End(builder: flatbuffers.Builder) -> uoffset: ...




# automatically generated by the FlatBuffers compiler, do not modify

# namespace: NamespaceC

import flatbuffers
from flatbuffers.compat import import_numpy

np = import_numpy()


class TableInC(object):
  __slots__ = ['_tab']

  @classmethod
  def GetRootAs(cls, buf, offset=0):
    n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
    x = TableInC()
    x.Init(buf, n + offset)
    return x

  @classmethod
  def GetRootAsTableInC(cls, buf, offset=0):
    """This method is deprecated. Please switch to GetRootAs."""
    return cls.GetRootAs(buf, offset)

  # TableInC
  def Init(self, buf, pos):
    self._tab = flatbuffers.table.Table(buf, pos)

  # TableInC
  def ReferToA1(self):
    o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
    if o != 0:
      x = self._tab.Indirect(o + self._tab.Pos)
      obj = TableInFirstNS()
      obj.Init(self._tab.Bytes, x)
      return obj
    return None

  # TableInC
  def ReferToA2(self):
    o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(6))
    if o != 0:
      x = self._tab.Indirect(o + self._tab.Pos)
      obj = SecondTableInA()
      obj.Init(self._tab.Bytes, x)
      return obj
    return None


def TableInCStart(builder):
  builder.StartObject(2)


def Start(builder):
  return TableInCStart(builder)


def TableInCAddReferToA1(builder, referToA1):
  builder.PrependUOffsetTRelativeSlot(
      0, flatbuffers.number_types.UOffsetTFlags.py_type(referToA1), 0
  )


def AddReferToA1(builder, referToA1):
  return TableInCAddReferToA1(builder, referToA1)


def TableInCAddReferToA2(builder, referToA2):
  builder.PrependUOffsetTRelativeSlot(
      1, flatbuffers.number_types.UOffsetTFlags.py_type(referToA2), 0
  )


def AddReferToA2(builder, referToA2):
  return TableInCAddReferToA2(builder, referToA2)


def TableInCEnd(builder):
  return builder.EndObject()


def End(builder):
  return TableInCEnd(builder)


try:
  from typing import Optional
except:
  pass


class TableInCT(object):

  # TableInCT
  def __init__(self):
    self.referToA1 = None  # type: Optional[TableInFirstNST]
    self.referToA2 = None  # type: Optional[SecondTableInAT]

  @classmethod
  def InitFromBuf(cls, buf, pos):
    tableInC = TableInC()
    tableInC.Init(buf, pos)
    return cls.InitFromObj(tableInC)

  @classmethod
  def InitFromObj(cls, tableInC):
    x = TableInCT()
    x._UnPack(tableInC)
    return x

  # TableInCT
  def _UnPack(self, tableInC):
    if tableInC is None:
      return
    if tableInC.ReferToA1() is not None:
      self.referToA1 = TableInFirstNST.InitFromObj(tableInC.ReferToA1())
    if tableInC.ReferToA2() is not None:
      self.referToA2 = SecondTableInAT.InitFromObj(tableInC.ReferToA2())

  # TableInCT
  def Pack(self, builder):
    if self.referToA1 is not None:
      referToA1 = self.referToA1.Pack(builder)
    if self.referToA2 is not None:
      referToA2 = self.referToA2.Pack(builder)
    TableInCStart(builder)
    if self.referToA1 is not None:
      TableInCAddReferToA1(builder, referToA1)
    if self.referToA2 is not None:
      TableInCAddReferToA2(builder, referToA2)
    tableInC = TableInCEnd(builder)
    return tableInC



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: Example

class Any(object):
    NONE = 0
    Monster = 1
    TestSimpleTableWithEnum = 2
    MyGame_Example2_Monster = 3

def AnyCreator(unionType, table):
    from flatbuffers.table import Table
    if not isinstance(table, Table):
        return None
    if unionType == Any.Monster:
        import MyGame.Example.Monster
        return MyGame.Example.Monster.MonsterT.InitFromBuf(table.Bytes, table.Pos)
    if unionType == Any.TestSimpleTableWithEnum:
        import MyGame.Example.TestSimpleTableWithEnum
        return MyGame.Example.TestSimpleTableWithEnum.TestSimpleTableWithEnumT.InitFromBuf(table.Bytes, table.Pos)
    if unionType == Any.MyGame_Example2_Monster:
        import MyGame.Example2.Monster
        return MyGame.Example2.Monster.MonsterT.InitFromBuf(table.Bytes, table.Pos)
    return None



# automatically generated by the FlatBuffers compiler, do not modify

# namespace: Example

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class StructOfStructs(object):
    __slots__ = ['_tab']

    @classmethod
    def SizeOf(cls):
        return 20

    # StructOfStructs
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # StructOfStructs
    def A(self, obj):
        obj.Init(self._tab.Bytes, self._tab.Pos + 0)
        return obj

    # StructOfStructs
    def B(self, obj):
        obj.Init(self._tab.Bytes, self._tab.Pos + 8)
        return obj

    # StructOfStructs
    def C(self, obj):
        obj.Init(self._tab.Bytes, self._tab.Pos + 12)
        return obj


def CreateStructOfStructs(builder, a_id, a_distance, b_a, b_b, c_id, c_distance):
    builder.Prep(4, 20)
    builder.Prep(4, 8)
    builder.PrependUint32(c_distance)
    builder.PrependUint32(c_id)
    builder.Prep(2, 4)
    builder.Pad(1)
    builder.PrependInt8(b_b)
    builder.PrependInt16(b_a)
    builder.Prep(4, 8)
    builder.PrependUint32(a_distance)
    builder.PrependUint32(a_id)
    return builder.Offset()

import MyGame.Example.Ability
import MyGame.Example.Test
try:
    from typing import Optional
except:
    pass

class StructOfStructsT(object):

    # StructOfStructsT
    def __init__(
        self,
        a = None,
        b = None,
        c = None,
    ):
        self.a = a  # type: Optional[MyGame.Example.Ability.AbilityT]
        self.b = b  # type: Optional[MyGame.Example.Test.TestT]
        self.c = c  # type: Optional[MyGame.Example.Ability.AbilityT]

    @classmethod
    def InitFromBuf(cls, buf, pos):
        structOfStructs = StructOfStructs()
        structOfStructs.Init(buf, pos)
        return cls.InitFromObj(structOfStructs)

    @classmethod
    def InitFromPackedBuf(cls, buf, pos=0):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, pos)
        return cls.InitFromBuf(buf, pos+n)

    @classmethod
    def InitFromObj(cls, structOfStructs):
        x = StructOfStructsT()
        x._UnPack(structOfStructs)
        return x

    # StructOfStructsT
    def _UnPack(self, structOfStructs):
        if structOfStructs is None:
            return
        if structOfStructs.A(MyGame.Example.Ability.Ability()) is not None:
            self.a = MyGame.Example.Ability.AbilityT.InitFromObj(structOfStructs.A(MyGame.Example.Ability.Ability()))
        if structOfStructs.B(MyGame.Example.Test.Test()) is not None:
            self.b = MyGame.Example.Test.TestT.InitFromObj(structOfStructs.B(MyGame.Example.Test.Test()))
        if structOfStructs.C(MyGame.Example.Ability.Ability()) is not None:
            self.c = MyGame.Example.Ability.AbilityT.InitFromObj(structOfStructs.C(MyGame.Example.Ability.Ability()))

    # StructOfStructsT
    def Pack(self, builder):
        return CreateStructOfStructs(builder, self.a.id, self.a.distance, self.b.a, self.b.b, self.c.id, self.c.distance)



from golden_utils import flatc_golden


def flatc(options, schema):
  # Wrap the golden flatc generator with C# specifics
  flatc_golden(options=["--csharp"] + options, schema=schema, prefix="csharp")


def GenerateCSharp():
  flatc([], "basic.fbs")
